; Created to support file selection from ENVI for
; mro_crism_destripe.pro
;
; Output files are written as BSQ.
;
; Created:  June, 2008
;           by Trevor Clark & Mario Parente
;
;Purpose:  To filter an image, removing spikes, surges, and stripes
;          Spikes:  Random pixels which differ drastically(by user input threshold)
;                   from their nearest neighbors in the spectral domain
;          Surges:  Horizontal lines of 'junk' pixels, which can be any values,
;                   positive, or negative, or 'bad' balues = 65535
;          Stripes: Vertical stripes, due to imperfect sensor calibration
;Operational Outline:
; HISTORY
; 06 May 2008    First version (after bugfix) supplied by Eldar and Mario
; 08 May 2008    Added wavelength_units to file query and passed to output header
; 23 jun 2008    Despike function modified to avoid memory problem by Mario and Trevor
; 27 jun 2008    Added surge to fix horizontal 'missing value' lines by Trevor
; 22 jul 2008    fixed averaging problem
; 11 aug 2008    'Bad' bands are no longer set to zero, they are fixed despite being 'bad'
; 12 aug 2008    added bad band list for short(VNIR), instead of using long(IR) for all
; 12 aug 2008    fixed widget errors which caused crash when values were changed, and made
;                use of 'just' despike or 'just' destripe possible
; 13 aug 2008    bad bands found with create_crism_bbl 
; 22 Sep 2008    option 'corr' hardcoded; corr = 1 allows correction to all bands, corr=0 corrects only good bands
; 22 Sep 2008    progress bar initializations corrected(purely aesthetic)
; 15 Dec 2008    Normalization filters added, despike made adaptive, based on standard dev. of local stats,
;                & widget updated
; 05 Feb 2009    Fixed destripe & despike output file selection to prevent crash
; 04 Dec 2009    FM - Use get_cat_header_data, transfer header info to 
;                processing routines in struct input_hdr, use integer type bbl.
; 17 Nov 2010    FM: -add header parameter CAT_PRODUCT_VERSION
; 18 Nov 2010    FM: -Four new CAT header parameters.


pro mro_crism_clean_envi, event

procedure_title = 'MRO CRISM CLEAN'

;Set compiler options
compile_opt strictarr

;Simple catch to prevent crash
catch, error
if (error NE 0) then begin
  cat_error_report, routine_name='mro_crism_clean_envi'
  return
endif

;Select input band from available bands list; disable subsetting (FM, 12/4/09):
envi_select, fid=in_fid, pos=in_pos, dims=in_dims, /no_dims, /no_spec, $
          title = procedure_title + ': Select Input File'
if (in_fid[0] eq -1) then begin
  print, procedure_title + ': Cancel/Invalid FID'
  return
endif

;Gather some useful info
stat = get_cat_header_data(in_fid, FNAME=in_filename, NS=ns, NL=nl, NB=nb, $
	WL=wl, WAVELENGTH_UNITS=wlunits, BBL=in_bbl, FWHM=fwhm,  $
	INTERLEAVE=interleave, OFFSET=offset, $
	DATA_IGNORE_VALUE=data_ignore_value, DATA_TYPE=data_type, $
	DEF_BANDS=def_bands, DESCRIP=descrip, $
	MAP_INFO=map_info, $
	XSTART=xstart, YSTART=ystart, $
	CAT_START_TIME          = cat_start_time, $
	CAT_SCLK_START          = cat_sclk_start, $
	CAT_CRISM_OBSID         = cat_crism_obsid, $
	CAT_OBS_TYPE            = cat_obs_type, $
	CAT_PRODUCT_VERSION     = cat_product_version, $
	CAT_CRISM_DETECTOR_ID   = cat_crism_detector_id, $
	CAT_BIN_MODE            = cat_bin_mode, $
	CAT_WAVELENGTH_FILTER   = cat_wavelength_filter, $
	CAT_CRISM_DETECTOR_TEMP = cat_crism_detector_temp, $
	CAT_CRISM_BENCH_TEMP    = cat_crism_bench_temp, $
	CAT_CRISM_HOUSING_TEMP  = cat_crism_housing_temp, $
	CAT_SOLAR_LONGITUDE     = cat_solar_longitude, $
	CAT_PDS_LABEL_FILE      = cat_pds_label_file, $
	CAT_SPECTRUM_RESAMPLED  = cat_spectrum_resampled, $
	CAT_TILE_WAVE_FILE      = cat_tile_wave_file, $
	CAT_SWEETSPOT_WAVE_FILE = cat_sweetspot_wave_file, $
	CAT_WA_WAVE_FILE        = cat_wa_wave_file,  $
	CAT_IR_WAVES_REVERSED   = cat_ir_waves_reversed, $
	CAT_HISTORY             = cat_history, $
	CAT_INPUT_FILES         = cat_input_files )

; Override in_pos to force processing of all bands:
in_pos = lindgen(nb)



;***************************************************
; Run widget program for user to choose corrections.
;***************************************************
TEMP:
    clean_widget,choices
    w=where(choices eq 'cancel')       ; cancel selected
    if (w[0] ne -1) then return
    destripe=choices[0]     ;options are 0 for noCleaned destripe, 1 for destripe
    gkernelsize=choices[1]  ;this should be a value
    gkernelsdx=choices[2]    ;this should be a value
    gkernelsdl=choices[3]    ;this should be a value
    despike=choices[4]      ;options are 0 for no despike, 1 for despike
    threshold=choices[5]    ;this should be a value

  
;***************************************************
; Run widget program for filename choice
;***************************************************
;Construct default output filenames
last_slash = strpos(in_filename, '/', /reverse_search) > strpos(in_filename, '\', /reverse_search)
last_dot = strpos(in_filename, '.', /reverse_search)
in_file_directory = strmid(in_filename, 0, last_slash + 1)
in_file_basename = strmid(in_filename, last_slash + 1, last_dot - last_slash -1)
if (last_dot NE -1) then begin
  in_file_extension = strmid(in_filename, last_dot)
endif else begin
  in_file_extension = ''
endelse

;;;;;;;;;;;DESTRIPE PROMPT;;;;;;;;;;;;;
if (destripe eq 1) then begin
 	output_file_default_destripe = in_file_directory + in_file_basename + '_destripe.img'

	;if you don't want to check with the user, and just write the default automatically, use this 
	;outfile_destripe = output_file_default_destripe  

	; DISABLE::: Check to see if user wants to write to a file or keep in memory:
	; Load into available bands list regardless.
	base = widget_auto_base(title='Destripe image output')
	wo = widget_outf(base, uvalue='outf',default=output_file_default_destripe, /auto)
	result = auto_wid_mng(base)
	if (result.accept eq 0) then return
	outfile_destripe = result.outf
	;mem_check = magic_mem_check(dims=in_dims, in_memory=result.outf.in_memory, $
    ;                out_dt=4, nb=nb, out_name=result.outf.name)
	;result.outf.name = mem_check.out_name
	;outfile_destripe = result.outf.name
	check = check_file_overwrite(outfile_destripe)
	if (check eq 0) then return
endif

;;;;;;;;;;;DESPIKE PROMPT;;;;;;;;;;;;;
if(despike eq 1) then begin
	if(destripe eq 1) then begin
		output_file_default_despike = in_file_directory + in_file_basename + '_destripe_despike.img'
	endif else begin
		output_file_default_despike = in_file_directory + in_file_basename + '_despike.img'
	endelse

	;if you don't want to check with the user, and just write the default automatically, use this 
	;outfile_despike = output_file_default_despike  

	; DISABLE::: Check to see if user wants to write to a file or keep in memory:
	; Load into available bands list regardless.
	base = widget_auto_base(title='Despike image output')
	wo = widget_outf(base, uvalue='outf',default=output_file_default_despike, /auto)
	;wo = widget_outfm(base, uvalue='outf',default=output_file_default_despike, /auto)
	result = auto_wid_mng(base)
	if (result.accept eq 0) then return
	outfile_despike = result.outf
	;mem_check = magic_mem_check(dims=in_dims, in_memory = result.outf.in_memory, $
    ;                out_dt=4, nb=nb, out_name = result.outf.name)
	;result.outf.name = mem_check.out_name
	;outfile_despike=result.outf.name
	check = check_file_overwrite(outfile_despike)
	if (check eq 0) then return
endif

;***************************************************
;  LOAD HEADER DATA INTO STRUCT
;***************************************************
input_hdr = { in_filename: in_filename, ns: ns, nl: nl, nb: nb, $
     wl: wl, wavelength_units: wlunits, bbl: in_bbl, $
     fwhm: fwhm, interleave: interleave, offset: offset, $
     data_ignore_value: data_ignore_value, data_type: data_type, $
     def_bands: def_bands, descrip: descrip, map_info: map_info, $
     cat_start_time: cat_start_time, $
     cat_sclk_start: cat_sclk_start, $
     cat_crism_obsid: cat_crism_obsid, $
     cat_obs_type: cat_obs_type, $
     cat_product_version: cat_product_version, $
     cat_crism_detector_id: cat_crism_detector_id, $
     cat_bin_mode: cat_bin_mode, $
     cat_wavelength_filter: cat_wavelength_filter, $
     cat_crism_detector_temp: cat_crism_detector_temp, $
     cat_crism_bench_temp: cat_crism_bench_temp, $
     cat_crism_housing_temp: cat_crism_housing_temp, $
     cat_solar_longitude: cat_solar_longitude, $
     cat_pds_label_file: cat_pds_label_file, $
     cat_spectrum_resampled: cat_spectrum_resampled, $
     cat_tile_wave_file: cat_tile_wave_file, $
     cat_sweetspot_wave_file: cat_sweetspot_wave_file, $
     cat_wa_wave_file: cat_wa_wave_file, $
     cat_ir_waves_reversed: cat_ir_waves_reversed, $
     cat_history: cat_history, $
     cat_input_files: cat_input_files }


;***************************************************
; Let the real thinking begin
;***************************************************
Start_Time = systime(1)
;print, 'Start Time: ', systime()


if(destripe eq 1) then begin
	cancel = destripe( ns, nl, nb, wl, input_hdr, outfile_destripe, $
	            procedure_title, gkernelsize, gkernelsdx, gkernelsdl, $
	            in_pos, in_dims, in_fid, in_bbl, xstart, ystart)
	if (cancel) then return  
	in_filename = outfile_destripe
endif

Destripe_Time = systime(1)
;print, 'Time at completion of Destripe: ', systime()

if(despike eq 1) then begin
   	void = despiker(ns, nl, nb, wl, in_filename, outfile_despike, $
	            procedure_title, gkernelsize, gkernelsdx, gkernelsdl, $
	            in_pos, in_dims, in_fid, in_bbl, threshold, spike_kernel_x, $
	            spike_kernel_s, robust_trim, xstart, ystart)
endif

;print, 'End Time  : ', systime()
End_Time = systime(1)
time = (Start_Time - End_Time)
if(destripe eq 1) then $
  print, 'Destrtipe runtime: ', (Destripe_Time - Start_Time)/60, ' minutes'
if(despike eq 1) then $
  print, 'Despike runtime: ', (End_Time - Destripe_Time)/60, ' minutes'
print, 'Total RunTime: ', (End_Time - Start_Time)/60, ' minutes'
END
