; ENVI event handler routine to make a z-profile using CRISM 
;  CDR4-WA/RW for wavelength information.  Can deal with 
;  PDS and CAT-ENVI files.
;
; RIGHT NOW, ONLY WORKS FOR TRDRs AND RTRs.
; ALSO, ONLY DOES ONE PLOT AT A TIME--CAN'T COLLECT SPECTRA
;
;  Note: hardcodeGNC = hardcoded for general naming conventions
;
; Created: Jan 30, 2006
; By: Shannon Pelkey
; Modification History:
; Aug 15,2006: SMP: Added check for single band files.
; Sep 12,2006: SMP: Updated to deal with flipped/masked files;
;                   does so by flipping WA/RW array if necessary 
;                   and forcing the value of the L detector
;                   crosstalk/first row to be original 
;                   wavelength value not CRISM NAN
; Oct 02,2006: SMP: Added check of compatibility of bin mode and wv 
;                   filter of CDR4-WA file with data.
; Nov 02,2006: SMP: Added simple error catch and updated routine usage.
; Feb 16,2007: SMP: Modified to create a .sav file
;                   --Changed use of 'sort' to 'bsort' to avoid issue
;                     on Windows.
; JUN 26,2007 SMP: --added check for tiles; aborts since not yet compatible.
; Aug 27,2008 MFM: --strupcase filenames in strpos() for case insensitivity
; Feb 10,2009 MFM: --For checking file types from filename contents, 
;                    use only file basename, don't include path.
; Mar 15,2009 MFM: --Filter 65535 values out of plotted data
; Sep 16,2009 MFM: --Replace find_cdr4file with find_cdr
; Sep 17,2009 MFM: --find_lblfile returns '' not '-1' on fail
; Nov 24,2010 MFM: --Use get_cat_header_data to get observation info
; Nov 24,2010 MFM: --Change find_cdr argument to fid
; Oct 07,2015 MFM: --Extensive modification to find WV wavelengths and work 
;                    with MTRDR/TER files. Also some other updates.


pro crism_zprofile_event, ev
;  widget_control, ev.id, get_uvalue=uvalue

  ; Define ENVI functions used so able to create a sav file:
  FORWARD_FUNCTION envi_get_slice,envi_pickfile


  ;Simple catch to prevent crash
  catch, error
  if (error NE 0) then begin
    cat_error_report, routine_name='crism_zprofile_event'
    return
  endif

  ;Get the display number
  widget_control, ev.top, get_uvalue=dn 
  
  ;Get the file ID for file open in this display:
  envi_disp_query,dn,fid=fid
  if (fid[0] eq -1) then return

  ; Get observation nfo needed
  stat = get_cat_header_data(fid, NS=ns, NL=nl, NB=nb, $
     WL=in_waves, WAVELENGTH_UNITS=wu, FNAME=datafile, $
     CAT_CRISM_DETECTOR_ID   = cat_crism_detector_id, $
     CAT_BIN_MODE            = cat_bin_mode, $
     CAT_WAVELENGTH_FILTER   = cat_wavelength_filter, $
     CAT_PDS_LABEL_FILE      = cat_pds_label_file, $
     CAT_SPECTRUM_RESAMPLED  = cat_spectrum_resampled, $
     CAT_SWEETSPOT_WAVE_FILE = cat_sweetspot_wave_file, $
     CAT_WA_WAVE_FILE        = cat_wa_wave_file,  $
     CAT_IR_WAVES_REVERSED   = cat_ir_waves_reversed)


  if (nb le 1) then begin
    ok = dialog_message('No CRISM Z profile for single band files.',$
                           title='CRISM Z-Profile Error')
    return
  endif
  dataupbase = strupcase(file_basename(datafile))

  ; Make sure not starting with tiles:
;  pos1 = strpos(dataupbase,'MRR')
;  if (pos1 ne -1) then begin
;    ok = dialog_message('crism_zprofile_event.pro not yet compatible with tiles.',$
;                         title='CRISM Z-Profile Error',/cancel)
;    return
;  endif

  ; Get wavelengths
  wvfile = find_wavelength_file(datafile)
  if (wvfile[0] eq '') then begin
  	ok = dialog_message('crism_zprofile_event: Cant find wavelength file for '+datafile, $
                         title='CRISM Z-Profile Error')
    return
  endif
  print,'Wavelengths: '+file_basename(wvfile)
  
  
  ; Is this a resampled format (tile or MTRDR)? If so there's only 
  ; one wavelength vector:
  extpos = strpos(wvfile, '.', /reverse_search)
  ext = strupcase(strmid(wvfile,extpos+1))
  if (ext eq 'TAB') then begin
    ; Find wavelength TAB file label:
    wvlbl = strmid(wvfile, 0, extpos) + '.LBL'
    wvlbl = file_search(wvlbl, /fold_case)
    if (wvlbl[0] eq '') then begin
  	  ok = dialog_message('crism_zprofile_event: Cant find label for wavelength file '+wvfile, $
                         title='CRISM Z-Profile Error')
      return
    endif
    read_crism_lbl, wvlbl, ['ROWS','COLUMNS'], val
    if (val[0] eq '') then begin
  	  ok = dialog_message('crism_zprofile_event: Cant get wavelength file size from '+wvlblfile,$
                         title='CRISM Z-Profile Error')
      return
    endif
    nr = fix(val[0])
    nc = fix(val[1])
    xx = fltarr(nc,nr)
    openr,u, wvfile, /get_lun
    readf,u, xx
    free_lun,u
    waves = xx[2,*]
    waves_2d = 0    ; Flag = 1-D vector
  endif else if (ext eq 'IMG') then begin
  	; 2-D wavelength file:
    find_lblfile, wvfile, wvlblfile
    if (wvlblfile[0] eq '') then begin
  	  ok = dialog_message('crism_zprofile_event: Cant find label for wavelength file '+wvfile, $
                         title='CRISM Z-Profile Error')
      return
    endif
    read_crism_lbl, wvlblfile, ['LINE_SAMPLES','BANDS'], val
    if (val[0] eq '') then begin
  	  ok = dialog_message('crism_zprofile_event: Cant get wavelength file size from '+wvlblfile,$
                         title='CRISM Z-Profile Error')
      return
    endif
    ns = fix(val[0])
    nb = fix(val[1])
    waves = fltarr(ns,nb)
    openr,u, wvfile, /get_lun
    readu,u, waves
    free_lun,u
    waves_2d = 1    ; Flag = 2-D vector
  endif else begin
    ok = dialog_message('crism_zprofile_event: Undefined wavelength file type '+wvfile,$
                         title='CRISM Z-Profile Error')
    return
  endelse


  ;;;;;;;;;;;;;;;;
  ; PLOT SPECTRUM:
  ;;;;;;;;;;;;;;;;
  ;Find cursor location for spectrum:
  disp_get_location, dn, xloc, yloc 

  col = xloc
  if (~waves_2d) then col = 0   ; Only 1-D wave table
  
  ; Get appropriate column and convert to microns.
  wavearr = reform(waves(col,*))
  k = where((wavearr lt 65000.0), nk)
  swap = 0
  if (nk gt 0) then begin
  	if (wavearr[k[0]] gt wavearr[k[nk-1]]) then swap = 1
  endif
  if (swap) then begin
  	wavearr = reverse(wavearr)
  endif
  nb = n_elements(wavearr)
  if (wavearr[nb-1] eq 65535.0) then wavearr[nb-1] = 4000.0
  wavearr = crism_waves2microns(wavearr)

;  ; See if we need to flip the wavelength dimension:
;  w=where(cat_bsort(wavearr) ne cat_bsort(in_waves))
;  if (w[0] ne -1) then wavearr=reverse(wavearr)

 ; ; Replace last band of L det arr with a 4. 
 ; ;  note: if there are more than one NAN, I do not
 ; ;        replace anything because it is a different 
 ; ;        situation
 ; wnan=where(wavearr eq 65.535)
 ; if (wnan[0] ne -1) and (n_elements(wnan) eq 1) then wavearr(wnan)=4.0
 

  ;Get a frame of data:
  spec=reform(envi_get_slice(fid=fid, line=yloc,xs=xloc,xe=xloc))
  if (swap) then spec = reverse(spec)

  ;Plot data:
  ; Find the platform CAT is running on:
  file = file_basename(datafile)
  k = where((abs(spec-65535.0) gt 0.1), nk)
  if (nk eq 0) then return
  envi_plot_data,wavearr[k],spec[k],$
            title='#'+strtrim((dn+1),2)+' CRISM Profile: '+file ,$ 
            xtitle='wavelength (microns)',ytitle='value',$
            plot_names=['x:'+strtrim(xloc+1,2)+' y:'+strtrim(yloc+1,2)]

end
