; ENVI event handler routine to read in CRISM files.
;
; Created: Jan 25, 2006
; By: Shannon Pelkey
;
; Subroutines used:  read_crism_lbl
;                    find_cdr
;
; Modification History:
; Mar 06,2006 SMP: --modified to handle request to expand file lacking
;                    associated .TAB file
; Jun 20,2006 SMP: --modified program to read in band names from DDR
;                    .LBL files and use CDR6 SW and WV tables to use
;                    the "sweet spot" wavelength array to load as default
;                    for data files
;                  --added SAMPLE_TYPE to LBL query; program marks
;                    anything with 'PC' in the type as byte_order=0
;                    (little endian), anything else is marked as
;                    byte_order=1 (big endian)
; Jul 25,2006 SMP: --removed option to "expand" cube
; Sep 06,2006 SMP: --Revamped procedure to look for returned rows in
;                    ROWNUM_TABLE object within the .IMG file; if
;                    cannot access the object successfully, uses old 
;                    procedure of getting row numbers from CDR6 WV 
;                    file and applying WV filter in obs. header
;                  --This change include a kludgey goto statement to
;                    handle the non-compliant CRISM-format OMEGA test
;                    data; hopefully I will remember to remove it once
;                    in PSP
; Oct 04,2006 SMP: --Added work around when ROWNUM_TABLE has
;                    nonsensical values.
; Nov 01,2006 SMP: --Added simple error catch;
; Feb 15,2007 SMP: --Fixed bug associated with non-PDS compliant file identification
; Feb 16,2007 SMP: --Modified to create a .sav file
; Mar 20,2007 SMP: --Modified to work with MRR tile files; revamped
;                    routine to handle all standard CRISM data products
; May 10,2007 SMP: --Added use of CDR6-BW for standard band widths.
;                    Uses just first column for both S and L
;                    detectors.
; Jun 28,2007 SMP: --Added simple mapinfo to tiles
; Aug 16,2007 MFM: --fixed crash on DDR read (Frank Morgan)
; Aug 17,2007 MFM: --corrected map info for tiles: now on CRISM MRDR projections
; Aug 23,2007 MFM: --convert nm -> um without changing 65535 (that was breaking atpcorr patch)
; Sep 07,2007 MFM: --add map info to tile MRRDE (backplane data)
; Oct 03,2007 MFM: --call file_is_cat to ID cat file instead of looking for _CAT 
;                    in filename; this prevents label search when opening CAT output 
;                    which is not always forced to have _CAT in name.
; Nov 16,2007 MFM: --cleaned up a handful of hardcoded unit numbers on openr's; use /get_lun
; May 08,2008 MFM: --use cat_file_check() to look for files; case insensitive to handle PDS files
; Aug 27,2008 MFM: --strupcase filenames in strpos() to make case insensitive.
; Jan 20,2009 MFM: --MRRDL gets band names
; Feb 10,2009 MFM: --For checking file types from filename contents, 
;                    use only file basename, don't include path.
; Mar 31,2009 MFM: --Use crism_waves2microns and crism_waves2nm for wavelength conversions
; Jul 09,2009 MFM: --Explicitly check cat_file_check return against 0.
; Jul 09,2009 MFM: --Include LDR (DDR for limb scan) in list of files with band names.
; Jul 27,2009 MFM: --Use load_tile_waves to get tile wavelengths, allowing PDS search
; Aug 21,2009 MFM: --Change load_tile_waves keyword to used_wvfile.
; Sep 16,2009 MFM: --Replace find_cdr6file with find_cdr
; Sep 17,2009 MFM: --find_lblfile returns '' not '-1' on fail. Deleted ENVI default open.
; Nov 19,2009 MFM: --Add bad bands to header setup. Use get_sw_waves to get 
;                    sweetspot waves and fwhm.
; Nov 27,2009 MFM: --Use get_cat_header_data; eliminates a lot of old code.
; Nov 27,2009 MFM: --Decrement def_bands before header setup.
; Nov 27,2009 MFM: --Replace goto,ender with 'continue'
; Dec 08,2009 MFM: --Check min(def_bands) instead of total(def_bands) 
; Jan 29,2010 MFM: --Get byte_order from get_cat_header_data, and send to 
;                    envi_setup_head (fixes EDR data corruption)
; Jan 29,2010 MFM: --Add lower case file filter.

pro read_crism_event, ev

widget_control, ev.id, get_uvalue=uvalue

; Define ENVI functions used so can create sav file:
FORWARD_FUNCTION envi_pickfile

; Simple catch to prevent crash:
catch, error
if (error NE 0) then begin
	cat_error_report, routine_name='read_crism_event'
	return
endif


; Select file:
fname = envi_pickfile(title='Select CRISM file(s) to open:',$
                      filter='*.img;*.IMG',/multiple_files)
if (fname[0] eq '') then return                ;cancel selected

for i=0,n_elements(fname)-1 do begin

	;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;; 
	; File does not need header info (ex. ADRs and CDRs) or
	;  is not original PDS. OPEN AS IS and skip:
	;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
	fnmupbase = strupcase(file_basename(fname[i]))
	pos1 = strpos(fnmupbase,'ADR')
	pos2 = strpos(fnmupbase,'CDR')
	not_pds = ~file_is_pds(fname[i])
	; Find a matching file, may not be same case
	fnd = cat_file_check(fname[i], files_found)
	if (fnd eq 0) then begin
		; This shouldn't happen...
		ok = dialog_message(file_basename(fname[i]) + ' not found.',$
                           title='CRISM File Not Found',/cancel)
		continue
	endif
	if ((pos1 ne -1) or (pos2 ne -1) or (not_pds)) then begin
		envi_open_file, files_found[0]
		continue
	endif


	;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
	; Get data for virtual ENVI header from label
	;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

	stat = get_cat_header_data(nofid, NS=ns, NL=nl, NB=nb, $
	         WL=wl, WAVELENGTH_UNITS=wu, BBL=bbl, FWHM=fwhm, $
	         BNAMES=bnames, INTERLEAVE=interleave, $
	         OFFSET=offset, BYTE_SWAP=byte_swap, BYTE_ORDER=byte_order, $
	         DATA_IGNORE_VALUE=data_ignore_value, DATA_TYPE=data_type, $
	         DEF_BANDS=def_bands, DESCRIP=descrip, $
	         MAP_INFO=map_info, $
	         LABEL_ONLY=fname[i] )
	if ((n_elements(def_bands) eq 3) && (min(def_bands) gt 0)) then begin
		; For header... see note in write_cat_header prolog about 
		; decrementing def_bands
		def_bands -= 1
	endif else begin
		def_bands = [0,0,0]
	endelse
	

	; Some checks
	if ( (ns lt 0) || $
	     (nl lt 0) || $
	     (nb lt 0) || $
	     (interleave lt 0) || $
	     (data_type lt 0) ) then begin
		ok = dialog_message('Required LBL info not found: use ENVI default opening procedure?',$
                           title='Read CRISM Message',/cancel)
		if (ok eq 'Cancel') then return
		envi_open_file,fname[i]
		continue
	endif

	envi_setup_head, fname=fname[i], ns=ns, nl=nl, nb=nb, BNAMES=bnames, $
	     interleave=interleave, data_type=data_type, byte_order=byte_order, $
	     offset=0, bbl=bbl, wavelength_unit=wu, wl=wl, fwhm=fwhm, $
	     data_ignore_value=65535.0, def_bands=def_bands, map_info=map_info, $
	     descrip='CRISM PDS File Imported into ENVI', /open

endfor                      ; for each file selected

return
end
