;*****************************************************************************
;
; function atm_correct_cube, wa, cube, vsid_in
;
; Given a CRISM scene cube, apply volcano scan correction 
; with artifact patch.
;
; INPUT:
;	cube      fltarr(ns,nb,nl)  Scene spectrum 
;	wa        fltarr(ns,nb)     Wavelengths
;	vstrans   fltarr(ns,nb)     Volcano scan transmission
;	vsart     fltarr(ns,nb)     Volcano scan artifact spectrum
;	iwave     fltarr(2)         Interpolation wavelengths
;	bandsetid int               0 -> McGuire scaling waves; 1-> Pelkey
;
; OUTPUT PARAMETERS:
;	errmsg     strarr(nl)       Error messages returned by patch_vs_artifact
;	zerodercnt intarr(nl)       vs_artifact_patch fit error counts each line
;	max_iter   intarr(nl)       vs_artifact_patch max iteration counts each line
;
; RETURN:
;	fltarr(ns,nb,nl) = Corrected cube
;
; KEYWORD:
;	NOART   If set, do NOT apply the artifact patch
;
; USES:
;	scaleatm_pcm
;	patch_vs_artifact
;
; NOTES:
;	Inputs can be spatially subsampled; but cube,wa,vstrans,vsart 
;	must all match.
;
; HISTORY
;	2012 Jul 13   FM  Created first version
;
;*****************************************************************************


function atm_correct_cube, cube, wa, vstrans, vsart, iwave, bandsetid, $
      		errmsg, zerodercnt, max_iter, noart=noart


; Dimensions:
s = size(cube)
ns = s[1]
nb = s[2]
if (s[0] eq 3) then begin
	nl = s[3]
endif else begin
	nl = 1
endelse

s = size(cube)
if (s[0] eq 2) then begin
	nl = 1
endif else begin
	nl = s[3]
endelse
if ((s[1] ne ns) or (s[2] ne nb)) then begin
	print, 'VS_CORRECT_CUBE: cube and WA dont match.'
	return, (cube*0.0) + 65535.0     ; all bad
endif

; output arrays...
corr = fltarr(ns,nb,nl) + 65535.0
errmsg = strarr(nl)
zerodercnt = intarr(nl)
max_iter = intarr(nl)


for i=0,nl-1 do begin
	frm = cube[*,*,i]
	; Locate bad pixels and if they're all bad skip this frame:
	kinf = where(~finite(frm), ninf)
	if (ninf gt 0) then frm[kinf] = 65535.0
	kbad = where((frm gt 65534.0), nbad, ncomplement=ngood)
	if (ngood eq 0) then continue
	
	; Apply volcano scan correction:
	scaleatm_pcm, frm, wa, vstrans, bandset=bandsetid
	
	; Reset any NaN or initially bad pixels = 65535:
	if (nbad gt 0) then frm[kbad] = 65535.0
	; and set any NaNs = 65535:
	wnan = where((~finite(frm)), ninf)
	if (ninf gt 0) then frm[wnan]=65535.
	; Get a new 65535 census:
	kbad = where((frm eq 65535.), nbad, ncomplement=ngood)
	if (ngood eq 0) then continue

	; Load first-order correction into output in case artifact patch fails:
	corr[*,*,i] = frm
	
	if (~keyword_set(noart)) then begin
		; Apply artifact correction
		stat = patch_vs_artifact(wa, frm, vsart, iwave, $
	             patched, xerrmsg, zero_deriv_count, max_iter_count)
		if (stat lt 0) then begin
			; Fatal error in artifact patch
			print, 'Fatal error in artifact patch line '+strtrim(i,2)+': ' + errmsg
		endif
		errmsg[i] = xerrmsg
		zerodercnt[i] = zero_deriv_count
		max_iter[i] = max_iter_count

		; Reset initially bad pixels = bad
		if (nbad gt 0) then patched[kbad] = 65535.0
		; and reset any new nans...
		wnan = where((~finite(patched)), ninf)
		if (ninf gt 0) then patched[wnan]=65535.

		; Dump the corrected frame into the corrected cube:
		corr[*,*,i] = patched
	endif
endfor

return, corr
end
