;******************************************************************************
;
; pro atp_tile, atp_setup
;
; Do atmospheric/thermal/photometric corrections for CRISM tile data.
;
; INPUT:
;	atp_setup    struct        Setup inputs for corrections
;	             .in_fid = Input ENVI file ID 
;	             .datafile = input data filename
;	             .in_dims = input file dimensions 
;	             .ns = number samples
;	             .nl = number lines
;	             .nb = number bands
;	             .in_interleave = interleave ('bsq' or 'bil')
;	             .in_wavesall = input wavelengths from envi header if found at open
;	                            (all wavelengths, regardless of subsampling)
;	             .wu = wavelength units (0: microns, 1: nm)
;	             .fwhm = sweet-spot bandwidths, from input header file, -1 if undefined there 
;	             .in_bands = indices into in_allwaves for selected subset bands
;	             .in_bbl = Bad bands list from input file
;	             .data_type = input data type (float)
;	             .start_type = 'RA' for radiance, 'IF' for I/F
;	             .resamp = resampled, 1; not, 0
;	             .subset = spectral subsetting requested, 1; not, 0
;	             .runall =run all wavelengths, 1; don't, 0
;	             .atm = Requested atmospheric correction: 'none', 'scale' or 'epf' 
;	             .therm = Requested thermal correction: 'none', 'algm', or 'algk'
;	             .phot = Requested photometric correction: 'on', 'off'
;	             .atmt_source = how to pick atmospheric transmission (default, auto, or user)
;	             .vsid_slct = selected volcano scan ID, if atmt_source = user or default
;	             .bandset_id = code for which wavelengths to scale to
;                .enable_artifact: flag to correct artifact or not
;	             .lblfile = PDS Label filename
;	             .memory = 1, request store result in memory; 0, store in file
;	             .outfile = output file name
;	             .def_bands = default ENVI display bands, invalid=[0,0,0]
;	             .map_info = map info structure
;	             .cat_product_version = cal version from PDS PRODUCT_VERSION_ID
;	             .cat_crism_detector_id = S, L, J
;	             .cat_bin_mode = CRISM bin mode code, 0,1,2,3
;	             .cat_wavelength_filter = CRISM wavelength filter code
;	             .cat_crism_detector_temp = detector temp, C
;	             .cat_crism_bench_temp = bench temp, C
;	             .cat_crism_housing_temp = housing temp, C
;	             .cat_solar_longitude = solar L_s
;	             .lblfile = PDS label file 
;	             .resamp = 1, data resampled; 0, not
;	             .cat_tile_wave_file = Wavelength file for tile data
;	             .cat_sweetspot_wave_file = SW CDR (nontile)
;	             .cat_wa_wave_file = WA CDR (nontile)
;	             .cat_ir_waves_reversed = 'YES' or 'NO'
;	             .cat_history = Previous CAT processing code string
;	             .cat_input_files = Previous CAT input files
;
; OUTPUT PARAMETERS: None.
;
; RETURN VALUE:  Status: 0, success; -1, failed or cancelled by user.
;
; SIDE EFFECT:
;	Writes file with corrected cube (if requested); opens to ENVI available bands.
;
; KEYWORDS: None.
;
; NOTES:
;
; USES:
;	cat_file_check
;	cat_tmpfile_name
;	crism_waves2nm
;	crism_photom_corr
;	load_tile_waves
;	read_crism_lbl
;	scaleatm_pcm
;	select_atmt_source
;	tile_support_fnm
;	write_bsq_slice
;	autolist_tile_cdr4s
;	autolist_tile_vsadr
;	pick_cdr_for_obs
;	get_cdr4_from_collection
;
; HISTORY:
;	2009 Jan 07  F.Morgan  First version. 
;	2009 Jan 07  MFM. Testing - on tile 00N088 compared to old atpcorr, 
;	             with photometric correction only, identical apart from 
;	             roundoff error.
;	2009 Jan 07  MFM. Added some temp file cleanup on fails/cancels.
;	2009 Jan 29  MFM  scaleatm_pcm replaces scaleatm, with scaling at 
;	                  user-selected bands.
;	2009 Feb 10  MFM  Added /reverse_search on a strpos for .IMG
;	2009 Feb 22  MFM  Check unit numbers before closing to prevent crash.
;	2009 Mar 31  MFM  Use crism_waves2microns for wavelength conversion
;	2009 Jul 09  MFM  Explicitly check cat_file_check return against 0.
;	2009 Jul 27  MFM  Update use of load_tile_waves; file search internal now.
;	2009 Aug 21  MFM  Add PDS tree search for WV filename from label.
;	2009 Sep 16  MFM  Documentation update
;	2009 Sep 17  MFM  Find_geofile returns '' not '-1' on fail
;	2009 Sep 18  MFM  Use load_tile_waves to load find file and wavelengths
;	2009 Sep 18  MFM  Add BATCH keyword. Make it a function returning status.
;	2009 Sep 21  MFM  Unify CDR loads for eith AT CDR or VS ADR, load list 
;	                  into at_list in either case, and get_cdr4_from_collection 
;	                  works with either source now. Got rid of tile_vsadr, 
;	                  no longer read VS ADR up front.  This corrects SCLK/version 
;	                  selection order problem since traces to new routine
;	                  select_dr and corrected routine vs_adr_filename.
;	2009 Nov 19  MFM  Add bad bands to output header. Use write_cat_header 
;                     to make header. Add FWHM to output header.
;                     Pass through parameters from input header.
;	2009 Nov 26  MFM  Eliminate byte_order check & keyword in write_cat_header.
;	2009 Dec 04  MFM  Check min(def_bands) instead of total(def_bands) 
;   2010 Nov 17  MFM  add header parameter CAT_PRODUCT_VERSION
;	2010 Dec 20  MFM  Added product version parameter and batch keyword 
;	                  to autolist_tile_* function calls. 
;	2010 Dec 20  MFM  Removed test print statements. 
;	2012 Mar 09  MFM  Add 'forward_function'
;	2012 Mar 23  MFM  Add volcano scan artifact patch. 
;	                  (TBD: get interp_waves and scale waves from label)
;   2015 Oct 07  MFM  Read and write MTRDR filtering flags to/from header
;
;******************************************************************************

function atp_tile, atp_setup, batch=batch


forward_function cat_file_check,cat_tmpfile_name,crism_waves2nm, $
         crism_photom_corr, load_tile_waves, tile_support_fnm, $
         write_bsq_slice, autolist_tile_cdr4s, autolist_tile_vsadr, $
         pick_cdr_for_obs, get_cdr4_from_collection

; Only ancillary data items not already supplied in atp_setup 
; are constant for tiles...
det = 'J'
wvflip = 'NO'


; Operation update... if scaling atm or doing thermal correction, 
; need to retrieve entire cube even if user has spectrally subset it:
if (atp_setup.atm ne 'none') or (atp_setup.therm ne 'none') then atp_setup.runall=1

;===========================
;   LOAD WAVELENGTHS
;===========================

; Load the wavelengths
stat = load_tile_waves(atp_setup.datafile, waves, rowidx, sensid, used_wvfile=used_wvfile)
if (stat ne 0) then return,-1    ; Failed to find wavelengths, and user canceled
; Convert to nm if needed
kgood = where((waves lt 65534.), ngood)
if (ngood eq 0) then begin
	if (~keyword_set(batch)) then begin
		msg = 'No valid wavelengths in '+used_wvfile
		ok = dialog_message(msg, title='Abort correction')
	endif
	return,-1
endif
waves = crism_waves2nm(waves)

; IR wavelengths and rows:
kir = where((sensid eq 0), tile_nir)
tile_irrow = rowidx[kir]       ; row numbers of IR wavelengths in tile
ir_waves = waves[kir]          ; IR wavelengths


;************************************************************************************
; BEGIN PROCESSING:  APPLY ATM, THERM, PHOT, CORR TO DATA:
;************************************************************************************
; If doing processing in memory, create a BSQ array to hold the data.
if (atp_setup.memory) then begin
	if (atp_setup.runall) then memdata = fltarr(atp_setup.ns, atp_setup.nl, atp_setup.nb) $
	else memdata = fltarr(atp_setup.ns, atp_setup.nl, n_elements(atp_setup.in_bands))
endif

; Setup default file mangement values:
outfiletmp1=''
outfiletmp2=''

; allbands for runall
allbands = indgen(atp_setup.nb)


;*************************
; PHOTOMETERIC CORRECTION:
;*************************
if (atp_setup.phot eq 'on') then begin

	; Find geometry file:
	find_geofile, atp_setup.lblfile, geofile, batch=batch
	if (geofile eq '') then return,-1    ; user canceled
	print, 'Using geometry file:'
	print, geofile

	; Check for compatibility: User may have selected the wrong file:
	; DEDUCE CDR LBL FILENAME AND CHECK FOR consistent binning/wavefilter:
	pos = strpos(strupcase(geofile),'.IMG',/reverse_search)
	if (pos eq -1) then begin
		geolbl = ''
	endif else begin
		geolbl=geofile
		strput,geolbl,'.LBL',pos
		fnd = cat_file_check(geolbl, geolbl_found)
		if (fnd eq 0) then begin
			geolbl = ''
		endif else begin
			geolbl = geolbl_found[0]
		endelse
	endelse
	if (geolbl ne '') then begin
		read_crism_lbl,geolbl,['LINES','LINE_SAMPLES'],geodims
		geonl=fix(strtrim(geodims[0],2))
		geons=fix(strtrim(geodims[1],2))
		if (geons ne atp_setup.ns) or (geonl ne atp_setup.nl) then begin
			if (~keyword_set(batch)) then begin
				ok = dialog_message('Spatial dimensions of '+geofile+' do not match data. Aborting.',$
    	                       title='Correct CRISM Error')
				if (ok eq 'OK') then return,-1
    		endif else begin
    			; Batch mode... return silently
    			return, -1
    		endelse
		endif
	endif else begin
		if (~keyword_set(batch)) then begin
			ok = dialog_message('Not enough LBL info to verify compatibility of '+geofile,$
                         title='Correct CRISM Message')
		endif
	endelse

	; Get the incidence angle data (with respect to the areoid)
	; (forcing dimensions)--in first plane for targeted obs, eleventh for tiles:
	openr,geo_lun, geofile, /get_lun, /swap_if_big_endian 
	assocde = ASSOC(geo_lun,fltarr(atp_setup.ns,atp_setup.nl,/nozero))      ; DDRs are BSQ
	iwrta = assocde[10] * !DTOR
	free_lun,geo_lun

	; Open file to write to if necessary:
	if (atp_setup.memory) then begin
		out_status_string = 'Output to Memory'
	endif else begin
		; if not ending here, need to write to temporary output file:
		if (atp_setup.atm eq 'none') and (atp_setup.therm eq 'none') then begin
			; Output to final output file
			openw, out_lun, atp_setup.outfile, /get_lun
			out_status_string = 'Output File: ' + atp_setup.outfile
			outfiletmp1 = ''     ; to flag tmp file not being used
		endif else begin
			; Outpt to tmp file
			outfiletmp1 = cat_tmpfile_name('tmp_corr','')
			openw, out_lun, outfiletmp1, /get_lun
			out_status_string = 'Output File: ' + outfiletmp1
		endelse
	endelse

	;Assemble string to be displayed in ENVI status window
	if (~keyword_set(batch)) then begin
		status_string_array = ['Applying Photometeric Correction: '  $
			+ atp_setup.datafile, out_status_string]
		envi_report_init, status_string_array, base = status_base, title = 'Correct CRISM', /interupt
		envi_report_inc, status_base, atp_setup.nl
	endif

	;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
	; Main processing loop: 
	; For tiles do one band at a time (so output will be BSQ)
	;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
	; Find out how many bands involved:
	if (~atp_setup.runall) then pullbands=atp_setup.in_bands else pullbands=allbands

	; Since processing a tile, can use incidence plane all at once:
	nloop = n_elements(pullbands)                   ; loop over bands

	; Loop through data:
	for j=0,nloop-1 do begin
		if (~keyword_set(batch)) then begin
			envi_report_stat, status_base, j, nloop, cancel=cancel
			if (cancel) then begin
				envi_report_init, base=status_base, /finish
				if ((~atp_setup.memory) && (n_elements(out_lun) gt 0)) then free_lun, out_lun
				if (outfiletmp1 ne '') then file_delete,outfiletmp1,/quiet
				return,-1
			endif
		endif

		; Get data plane from input file for current band.
		dataframe = envi_get_data(fid=atp_setup.in_fid, pos=pullbands[j], $
					dims=[-1,0,atp_setup.ns-1,0,atp_setup.nl-1])

		; Correct the image:
		stat = crism_photom_corr(dataframe, iwrta, plane_corr)
		if (stat ne 0) then begin
			print, 'ATP NONTILE:  Photometric correction dimensions invalid.'
		endif

		; Store output in correct location; output is BSQ
		if (~atp_setup.memory) then begin
			writeu, out_lun, plane_corr
		endif else begin
			memdata[*,*,j] = plane_corr
		endelse
	endfor
	if (~keyword_set(batch)) then envi_report_init, base=status_base, /finish
	free_lun,out_lun
	if ((~atp_setup.memory) and (outfiletmp1 ne '')) then begin
		; Give the temp file a basic header so can open for envi access later.
		envi_setup_head, data_type=4, fname=outfiletmp1, nb=n_elements(pullbands), $
		           ns=atp_setup.ns, nl=atp_setup.nl, interleave=0, offset=0, /write
	endif
endif


;********************
; THERMAL CORRECTION:
;********************
; If no phot corr:
;   Need to get data from input file.
; If phot corr on:
;   Need to get data from array in memory or temporary file.
if (atp_setup.therm ne 'none') then begin

	; Input 
	if (~atp_setup.memory) then begin
		; Open a BSQ file for input data
		if (outfiletmp1 ne '') then begin
			; Read from the phot-corr temp file output, outfiletmp1
			envi_open_file, outfiletmp1, /no_realize, r_fid=therm_in_id
		endif else begin
			; Read from original input file
			therm_in_id = atp_setup.in_fid
		endelse
	endif

	; Output
	if (atp_setup.memory) then begin
		out_status_string = 'Output to Memory'
	endif else begin
		if (atp_setup.atm eq 'none') then begin
			; This is the last correction, write to final output BSQ file.
			openw, out_lun, atp_setup.outfile, /get_lun, /swap_if_big_endian
			out_status_string = 'Output File: ' + atp_setup.outfile
		endif else begin
			; Write to a BSQ temp file which will be read as input for the 
			; atmspheric correction
			outfiletmp2 = cat_tmpfile_name('therm_corr','')
			openw, out_lun, outfiletmp2, /get_lun
			out_status_string = 'Output File: ' + outfiletmp2
		endelse
	endelse


	; pick algorithm:
	if (therm eq 'algm') then begin

	endif                           ; close empirical therm correction

	if (therm eq 'algk') then begin

	endif                           ; close modeled therm correction

	; Remember to subset the data if not continuing to do atm correction:
	;  if (atm eq 'none') and (subset eq 1) and (runall eq 1) then begin
	;    runall=0
	;    if (memory eq 1) memdata=memdata(*,*,in_bands)
	;    ; If writing to files, then only write out necessary bands above
	;  endif

	    ; Write output to BSQ file at 'out_lun' using write_bsq_slice.pro 
    	; and then recreate a simple header. (like in atm correction below.)
		; out_lun already opened above, either final output or new temp file.

	
	free_lun,out_lun
	if ((~atp_setup.memory) and (outfiletmp2 ne '')) then begin
		; Give the temp file a basic header so can open for envi access later.
		envi_setup_head, data_type=4, fname=outfiletmp2, nb=atp_setup.nb, $
		           ns=atp_setup.ns, nl=atp_setup.nl, interleave=0, offset=0, /write
	endif

	; Clean up photometric correction temp file if created; no longer needed.
	if (outfiletmp1 ne '') then begin
		envi_file_mng, id=therm_in_id, /remove, /delete
		file_delete, outfiletmp1+'.hdr', /quiet
	endif
	outfiletmp1=''

endif              ; ENDOF THERMAL CORRECTION


patpix = bytarr(1270,1280)
;*************************
; ATMOSPHERIC CORRECTION:
;*************************
; If no phot or therm corr: Need to get data from input file.
; If phot and/or therm corr on: Need to get data from array in memory or temporary file.

if (atp_setup.atm ne 'none') then begin

	;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
	; If scaling, need to get atmorap and wavelength array
	;  prior to processing:
	;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
	if (atp_setup.atm eq 'scale') then begin


		;======================================================
		;              LOAD ATM TRANSMISSION
		;======================================================

		;-----------------------------------------------
		;  Get data for picking spectrum for each pixel
		;-----------------------------------------------

		; Need column numbers and, if auto-selecting time-dependent, 
		; wavelength shift optimized AT CDR, need SCLK for each location. 
		; To get SCLK, need observation ID. Column numbers and obsID 
		; come from MRRDE...
		defile = tile_support_fnm(atp_setup.datafile, 'MRRDE')
		; User had a chance to search in tile_support_fnm, 
		; if not found, give up:
		if (defile eq '') then begin 
			if (outfiletmp1 ne '') then file_delete,outfiletmp1,/quiet
			if (outfiletmp2 ne '') then file_delete,outfiletmp2,/quiet
			return,-1
		endif

		; MRRDEs are BSQ
		openr,de_lun, defile, /get_lun, /swap_if_big_endian 
		assocde = assoc(de_lun,fltarr(atp_setup.ns,atp_setup.nl,/nozero))
		irid = assocde[3]                    ; Obs ID
		ircol = assocde[7]                   ; Column number
		free_lun, de_lun


		;-----------------------------------------------
		;    SELECT PIXEL-DEPENDENT CDRs
		;-----------------------------------------------

		; For CDR selection (WA, AT)...
		;  * wavelength filter = 1 for IR MSP (assume always relevant to tiles)
		;  * bin code = 3 (10 pixel) for MSP (assume always relevant to tiles)
		; ** NOTE: These assumptions are good for now (confirmed by Chris Hash, 9/18/09) 
		; **       BUT if MSW's ever get incorporated into the tiles, THEY WILL FAIL.
		wave_filt = 1
		bincode = 3

		; AT CDR, or VS ADR

		if (atp_setup.atmt_source eq 'auto') then begin

			; AT and WA CDRs are both pixel dependent.  For now, just
			; auto-select the AT CDRs. (Wait till later for WA, so only have 
			; to code it once; will need auto-selected WA regardless.)

			stat = autolist_tile_cdr4s(irid, ['AT'], 'L', $
			           bincode, wave_filt, atp_setup.cat_product_version, $
			           at_list, nfail, batch=batch)
			if (stat ne 0) then begin
				if (~keyword_set(batch)) then begin
					msg = 'CDR autolist failed.'
					ok = dialog_message(msg, title='Aborting Correction')
				endif
				if (outfiletmp1 ne '') then file_delete,outfiletmp1,/quiet
				if (outfiletmp2 ne '') then file_delete,outfiletmp2,/quiet
				return,-1
			endif
			if (nfail[0] gt 0) then begin
				; Failed to find some AT CDRs
				if (~keyword_set(batch)) then begin
					; Interactive mode... ask user if they want to quit
					msg = 'Failed to autoselect AT CDR for ' + strtrim(nfail[0],2) + ' MSPs.' $
				        + string([10b]) $
				        + 'Would you like to use a different volcano scan selection?' $
				        + string([10b]) $
				        + '(If not correction will skip tile pixels using those MSP.)'
					ok = dialog_message(msg, title='AT Auto-Select Failed',/cancel,/question)			
					if (ok eq 'Yes') then begin
						; User approved changing AT selection method, but disallow auto:
						select_atmt_source, atmt_src, vsid, /noauto
						atp_setup.atmt_source = atmt_src
						atp_setup.vsid_slct = vsid
					endif
				endif
				; If batch mode, just push on and 65535 the pixels where CDR find failed.
			endif
		endif
		
		if (atp_setup.atmt_source eq 'tbench') then begin			

			; Select VS ADR's based on the volcano scan that best matches the 
			; optical bench temperature dependent spectral shift. Also sclk 
			; dependent of course.

			; Call with vsid = -1 tells autolist to use bench temp 
			; selection instead of using same VSID for all obs:
			stat = autolist_tile_vsadr(irid, -1, bincode, wave_filt, $
			          atp_setup.cat_product_version, at_list, nfail, batch=batch)
			if (stat ne 0) then begin
				atp_setup.fail_message = 'VS ADR autolist failed.'
				if (~keyword_set(batch)) then begin
					ok = dialog_message(atp_setup.fail_message, title='Aborting Correction')
				endif
				if (outfiletmp1 ne '') then file_delete,outfiletmp1,/quiet
				if (outfiletmp2 ne '') then file_delete,outfiletmp2,/quiet
				return,-1
			endif
			if (nfail[0] gt 0) then begin
				; Failed to find some AT CDRs
				if (~keyword_set(batch)) then begin
					; Interactive mode... ask user if they want to quit
					msg = 'Failed to autoselect VS ADR for ' + strtrim(nfail[0],2) + ' MSPs.' $
				        + string([10b]) $
				        + 'Would you like to use a different volcano scan selection?' $
				        + string([10b]) $
				        + '(If not correction will skip tile pixels using those MSP.)'
					ok = dialog_message(msg, title='AT Auto-Select Failed',/cancel,/question)			
					if (ok eq 'Yes') then begin
						; User approved changing AT selection method, but disallow auto:
						select_atmt_source, atmt_src, vsid, /noTbench
						atp_setup.atmt_source = atmt_src
						atp_setup.vsid_slct = vsid
					endif
				endif
				; If batch mode, just push on and 65535 the pixels where CDR find failed.
			endif
		endif
		
		; Can't just else into here, because might have just switched to a different 
		; method inside the auto block just above...

		if ((atp_setup.atmt_source ne 'auto') and (atp_setup.atmt_source ne 'tbench')) then begin
			; Will get transmission spectrum from a single specified volcano scan
			; for the whole tile. BUT - the VS ADR is pixel-dependent anyway, 
			; because of the SCLK dependence (through wavelength tables).
			; So... list VS ADRs into variable at_list... then can just pick 
			; a data record from at_list regardless of whether it's an 
			; AT CDR or a VS ADR.
			stat = autolist_tile_vsadr(irid, atp_setup.vsid_slct, bincode, $
			                     wave_filt, atp_setup.cat_product_version, $
			                     at_list, nfail, batch=batch)
			if (stat ne 0) then begin
				if (~keyword_set(batch)) then begin
					msg = 'VS ADR autolist failed for VSID ' + strtrim(atp_setup.vsid_slct,2)
					ok = dialog_message(msg, title='Abort Correction')
				endif
				if (outfiletmp1 ne '') then file_delete,outfiletmp1,/quiet
				if (outfiletmp2 ne '') then file_delete,outfiletmp2,/quiet
				return,-1
			endif
			if (nfail gt 0) then begin
				; Failed to find some VS ADRs
				if (~keyword_set(batch)) then begin
					; Interactive mode... ask user if they want to quit
					msg = 'Failed to list VS ADR for ' + strtrim(nfail,2) + ' MSPs.' $
					     + string(10b) $
					     + 'Correction will skip affected tile pixels and insert 65535.'
					ok = dialog_message(msg, title='AT Auto-Select Failed')			
				endif
				; If batch mode, just push on and 65535 the pixels where CDR find failed.
			endif 
		endif


		; WA CDR  (Always "auto-select" WA CDR)
		stat = autolist_tile_cdr4s( irid, ['WA'], 'L', $
		           bincode, wave_filt, atp_setup.cat_product_version, $
		           wa_list, nfail, batch=batch)
		if (nfail[0] gt 0) then begin
			; Failed to find some WA CDRs
			if (~keyword_set(batch)) then begin
				; Interactive mode... ask user if they want to quit
				msg = 'Failed to autoselect WA CDR for ' + strtrim(nfail[0],2) + ' MSPs.' $
		    	    + string([10b]) $
		    	    + 'Use nominal MRRWV tile wavelengths for pixels covered by those?' $
			        + string([10b]) $
			        + '(Uncompensated smile may cause gasband artifacts.)'
				ok = dialog_message(msg, title='WA Auto-Select Failed',/cancel,/question)
			endif
			if (ok eq 'Yes') then begin
				k = where((wa_list.fnm[0] eq ''), nk)
				if (nk gt 0) then wa_list[k].fnm[0] = 'MRRWV'
			endif
			; If batch mode, just push on and 65535 the pixels where CDR find failed.

			; So: Where wa_list.fnm = '', correction = 65535. 
			;     where wa_list.fnm = 'MRRWV' use tile wavelengths
			;     Everywhere else use WA CDR to get real wavelengths
		endif


		; Now, CDR filename lists are wa_list, and at_list


		;-----------------------------
		;  INPUT AND OUTPUT
		;-----------------------------

		; Open input file if not from memory.
		if (~atp_setup.memory) then begin
			; Open a BSQ file for input data
			atm_in_tmp = ''
			if (outfiletmp2 ne '') then begin
				; Read from the therm-corr temp file output, outfiletmp2
				atm_in_tmp = outfiletmp2
				envi_open_file, atm_in_tmp, /no_realize, r_fid=atm_in_id
			endif else if (outfiletmp1 ne '') then begin
				; Read from the phot-corr temp file output, outfiletmp1
				atm_in_tmp = outfiletmp1
				envi_open_file, atm_in_tmp, /no_realize, r_fid=atm_in_id
			endif else begin
				; Read from original input file
				atm_in_id = atp_setup.in_fid
			endelse
		endif

		;-----------------------------
		;  APPLY CORRECTION
		;-----------------------------

		; scaling wavelengths...
		if (atp_setup.enable_artifact) then begin
			case atp_setup.bandset_id of
				0: scale_waves = [2007.0,1980.0]
				1: scale_waves = [2011.0,1899.0]
				else: begin
					atp_setup.fail_message = 'Unsupported scaling bandset: '+strtrim(atp_setup.bandset_id,2)
					if (strlen(strtrim(tmpfile)) gt 0) then file_delete,tmpfile,/quiet
					return, -1
				end
			endcase
		endif

		; Open output file to write to if necessary:
		if (atp_setup.memory) then begin
			out_status_string = 'Output to Memory'
		endif else begin
			; There are no further corrections, so write to final output BSQ file:
			openw, out_lun, atp_setup.outfile, /get_lun, /swap_if_big_endian
			out_status_string = 'Output File: ' + atp_setup.outfile
		endelse

		;Assemble string to be displayed in ENVI status window
		if (~keyword_set(batch)) then begin
			status_string_array = ['Applying Atmospheric Correction to ' $
					+ atp_setup.datafile, out_status_string]
			envi_report_init, status_string_array, base=status_base, title='Correct CRISM', /interupt
			envi_report_inc, status_base, atp_setup.nl
		endif

		; Main processing loop: one line at a time using all bands
		npatch = 0L     ; count number of pixels successfully artifact-patched
		ntot = 0L       ; count number of valid pixels patched or not
		for j=0,atp_setup.nl-1 do begin
			if (~keyword_set(batch)) then begin
				envi_report_stat, status_base, j, atp_setup.nl, cancel=cancel
				if (cancel) then begin
					; User canceled, quit and clean up
					stat = get_cdr4_from_collection(/kill)    ; clear the CDR heap
					envi_report_init, base = status_base, /finish
					if ((~atp_setup.memory) && (n_elements(out_lun) gt 0)) then free_lun, out_lun
					if (atm_in_tmp ne '') then begin
						if (n_elements(atm_in_id) gt 0) then begin
							envi_file_mng, id=atm_in_id, /remove, /delete
						endif
						file_delete, atm_in_tmp+'.hdr', /quiet
					endif
					if (outfiletmp1 ne '') then file_delete,outfiletmp1,/quiet
					if (outfiletmp2 ne '') then file_delete,outfiletmp2,/quiet
					return,-1
				endif
			endif

			; Grab a BIL-like slice of data, ns x nb, from BSQ memory or file.
			if (atp_setup.memory eq 1) then begin
				dataframe = reform(memdata[*,j,*])
			endif else begin
				dataframe = envi_get_slice(fid=atm_in_id, line=j, pos=allbands, $
				                xs=0, xe=atp_setup.ns-1)
			endelse


			; Index the NaNs in the dataframe
			wnan = where((  (abs(dataframe-65535.) lt 0.1)  $
			             or (~finite(dataframe)) ), n_nan)

			; For tiles, current version of scaleatm wants 2-D data slice ns x nb, 
			; 1-D wavelength vector, and 1-D transmission spectrum.
			; dataframe = 2-D data slice, but different samples potentially see 
			; different atmospheres because they come from different observations 
			; within the tile. So, have to do a single sample at a time. For each, 
			; pull WC and AT CDR from collection, get the wavelengths and transmission 
			; spectrum for the appropriate column, and send a 1 x nir data to 
			; scaleatm.
			;
			; Also, only send scaleatm the IR part of the spectrum. The nominal 
			; tile wavelengths for the IR spectrum are ir_waves (from way at the top).
			; And the 'kir' are indices into the tile bands for the IR bands.

			for i=0,atp_setup.ns-1 do begin

				;-------------------------
				;   CHECK PIXEL VALIDITY
				;-------------------------
				if (irid[i,j] eq 65535L) then begin
					dataframe[i,*] = 65535.0
					continue       ; skip rest of this iteration
				endif

				;---------------------
				;   IR DATA SPECTRUM
				;---------------------
				; Pull the IR part of the data at this tile pixel from dataframe.
				d_ir = dataframe[i, kir]      ; dimension 1 x tile_nir


				;---------------------
				;   WAVELENGTHS
				;---------------------

				wa_fnm = pick_cdr_for_obs(irid[i,j], wa_list, 0)
				if (wa_fnm eq '') then begin
					; Failed to select a WA CDR, and user didn't allow tile wavelengths,
					; so can't correct at this location:
					d_ir[0,*] = 65535.0
				endif else if (wa_fnm eq 'MRRWV') then begin
					; User is allowing tile wavelengths
					waves_here = ir_waves
				endif else begin
					; Get IR wavelengths from WA collection; get_cdr4_from_collection
					; will pull the appropriate CDR from the heap if already loaded, or
					; load it if not.
					stat = get_cdr4_from_collection(wa_fnm, waimg, wa_rowidx)
					if (stat ne 0) then begin
						; No match at all between CDR and tile rows.
						if (~keyword_set(batch)) then begin
							msg = 'Failed to load WA CDR ' + wa_fnm $
						     + string([10b]) + 'Atmospheric correction failed.' $
						     + string([10b]) + 'get_cdr4_from_collection() code '+strtrim(stat)
							ok = dialog_message(msg, title='Abort correction')
						endif
						stat = get_cdr4_from_collection(/kill)    ; clear cdrs off the heap 
						if (atm_in_tmp ne '') then begin
							if (n_elements(atm_in_id) gt 0) then begin
								envi_file_mng, id=atm_in_id, /remove, /delete
							endif
							file_delete, atm_in_tmp+'.hdr', /quiet
						endif
						if (outfiletmp1 ne '') then file_delete,outfiletmp1,/quiet
						if (outfiletmp2 ne '') then file_delete,outfiletmp2,/quiet
						return,-1
					endif						
					; Pull the wavelengths for the current column from this WA
					wa_col = transpose(waimg[ircol[i,j], *])
					; And pull the appropriate bands and reorder to match tile rows:
					stat = cdrspec_for_tile(wa_rowidx, wa_col, tile_irrow, waves_here)
					if (stat ne 0) then begin
						; No match at all between CDR and tile rows. Fail
						if (~keyword_set(batch)) then begin
							msg = 'No match between WA CDR and tile rows.' $
						        + string([10b]) + 'Atmospheric correction failed.'
							ok = dialog_message(msg, title='Abort correction')
						endif
						stat = get_cdr4_from_collection(/kill)    ; clear cdrs off the heap 
						if (atm_in_tmp ne '') then begin
							if (n_elements(atm_in_id) gt 0) then begin
								envi_file_mng, id=atm_in_id, /remove, /delete
							endif
							file_delete, atm_in_tmp+'.hdr', /quiet
						endif
						if (outfiletmp1 ne '') then file_delete,outfiletmp1,/quiet
						if (outfiletmp2 ne '') then file_delete,outfiletmp2,/quiet
						return,-1
					endif						
				endelse
				; Now either waves_here = the wavelength array for this tile pixel, 
				; or no wavelengths found and d_ir[i,j]=65535.


				;--------------------------------
				;   ATMOSPHERIC TRANSMISSION
				;--------------------------------
				
				; Pull the transmission for this tile pixel and resort it into 
				; tile bands, just like for WA. (Regardless of whether transmission 
				; was autoselected or not, listed eith AT CDR's or VS ADR's into 
				; at_list, so now, pull appropriate filename for the MSP at hand, 
				; and load it.)

				at_fnm = pick_cdr_for_obs(irid[i,j], at_list, 0)  
				if (at_fnm eq '') then begin
					; Failed to select a AT CDR, so can't correct at this location:
					d_ir[0,*] = 65535.0
				endif else begin
					; Get IR transmission from AT collection; get_cdr4_from_collection
					; will pull the appropriate CDR from the heap if already loaded, or
					; load it if not.
					; Don't worry about checking result; next will fail if problem.
					stat = get_cdr4_from_collection(at_fnm, atimg, at_rowidx)
					if (stat ne 0) then begin
						; No match at all between CDR and tile rows.
						if (~keyword_set(batch)) then begin
							msg = 'Failed to load WA CDR ' + at_fnm $
						     + string([10b]) + 'Atmospheric correction failed.' $
						     + string([10b]) + 'get_cdr4_from_collection() code '+strtrim(stat)
							ok = dialog_message(msg, title='Abort correction')
						endif
						stat = get_cdr4_from_collection(/kill)    ; clear cdrs off the heap 
						if (atm_in_tmp ne '') then begin
							if (n_elements(atm_in_id) gt 0) then begin
								envi_file_mng, id=atm_in_id, /remove, /delete
							endif
							file_delete, atm_in_tmp+'.hdr', /quiet
						endif
						if (outfiletmp1 ne '') then file_delete,outfiletmp1,/quiet
						if (outfiletmp2 ne '') then file_delete,outfiletmp2,/quiet
						return,-1
					endif
					; Pull the wavelengths for the current column from this WA
					at_col = transpose(atimg[ircol[i,j], *,0])

					; For now, for tiles, assume bandset 0 = artifact 1, 
					; bandset 1 = artifact 2...

					do_art = 0     ; flag whether to apply artifact correction
					if ((atp_setup.enable_artifact) and ((size(atimg))[0] eq 3)) then begin
						do_art = 1     ; correct artifact on this pixel
						if (atp_setup.bandset_id eq 0) then begin
							artifact_col = transpose(atimg[ircol[i,j],*,1])
						endif else if (atp_setup.bandset_id eq 1) then begin
							artifact_col = transpose(atimg[ircol[i,j],*,2])
						endif else begin
							do_art = 0
						endelse
					endif
	
					; And pull the appropriate bands and reorder to match tile rows:
					stat = cdrspec_for_tile(at_rowidx, at_col, tile_irrow, atm_trans)
					if (stat ne 0) then begin
						; No match at all between CDR and tile rows.
						if (~keyword_set(batch)) then begin
							msg = 'No match between AT CDR and tile rows.' $
					    	    + string([10b]) + 'Atmospheric correction failed.'
							ok = dialog_message(msg, title='Abort correction')
						endif
						stat = get_cdr4_from_collection(/kill)    ; clear cdrs off the heap 
						if (atm_in_tmp ne '') then begin
							if (n_elements(atm_in_id) gt 0) then begin
								envi_file_mng, id=atm_in_id, /remove, /delete
							endif
							file_delete, atm_in_tmp+'.hdr', /quiet
						endif
						if (outfiletmp1 ne '') then file_delete,outfiletmp1,/quiet
						if (outfiletmp2 ne '') then file_delete,outfiletmp2,/quiet
						return,-1
					endif						

					; Same for the artifact...
					if (do_art) then begin
						stat = cdrspec_for_tile(at_rowidx, artifact_col, tile_irrow, atm_art)
						if (stat ne 0) then begin
							; No match at all between CDR and tile rows.
							if (~keyword_set(batch)) then begin
								msg = 'No match between AT CDR and tile rows.' $
						    	    + string([10b]) + 'Atmospheric correction failed.'
								ok = dialog_message(msg, title='Abort correction')
							endif
							stat = get_cdr4_from_collection(/kill)    ; clear cdrs off the heap 
							if (atm_in_tmp ne '') then begin
								if (n_elements(atm_in_id) gt 0) then begin
									envi_file_mng, id=atm_in_id, /remove, /delete
								endif
								file_delete, atm_in_tmp+'.hdr', /quiet
							endif
							if (outfiletmp1 ne '') then file_delete,outfiletmp1,/quiet
							if (outfiletmp2 ne '') then file_delete,outfiletmp2,/quiet
							return,-1
						endif
					endif
				endelse


				;--------------------------------
				;   APPLY CORRECTION
				;--------------------------------
				; Now finally can correct the data. Apply volcano scan correction,
				; replacing 65535 afterwards:
				kbadir = where((abs(d_ir-65535.) lt 0.1), nbadir, ncomplement=ngoodir)
				if (ngoodir eq 0) then begin
					dataframe[i,*] = 65535.0
					continue
				endif
				scaleatm_pcm, d_ir, waves_here, atm_trans, bandset=atp_setup.bandset_id
				if (nbadir gt 0) then d_ir[kbadir]=65535.

				if (do_art) then begin
					; Apply artifact patch; atm_art = artifact spectrum; need 
					; to make sure all inputs are 2D even though there's 
					; only 1 row in play, or subscripting in patch_vs_artifact 
					; is wrong...
					waves_here = reform(waves_here, 1, n_elements(waves_here))
					atm_art = reform(atm_art, 1, n_elements(atm_art))
					; Remember where bad pixels are in any inputs...
					kbad = where((   (d_ir gt 65534.0)        $
					              or (waves_here gt 65534.0)  $
					              or (atm_art gt 65534.0)  ), nbad)
					; For now, for tiles, assume interpolation waves = 1764.0,2239.0
					interp_waves = [1764.0,2239.0]

					stat = patch_vs_artifact(waves_here, d_ir[0,*], atm_art, interp_waves, $
					     patched, errmsg, zero_deriv_count, max_iter_count)
					if (stat eq 0) then begin
						; Put back the bad value flags:
						if (nbad gt 0) then patched[kbad]=65535.0
						; And put the patched spectrum into corrected tile:
						dataframe[i,kir] = patched[*]
						npatch++
						patpix[i,j]=1B
					endif
				endif else begin
					; Else, patch didn't work for this pixel, put the unpatched,
					; VS corrected spectrum back in the IR pixels of dataframe:
					dataframe[i,kir] = d_ir[0,*]
				endelse
			endfor       ; end 'ns' loop over samples

			; Put original NaNs back:
			if (n_nan gt 0) then dataframe[wnan] = 65535.

			; Take care of any other NANs:
			wnan = where((finite(dataframe) eq 0.), n_bad)
			if (n_bad gt 0) then dataframe[wnan] = 65535.

			; And write the corrected dataframe to appropriate location:
			if (atp_setup.memory) then begin
				memdata(*,j,*) = dataframe
			endif else begin
				if ((atp_setup.subset) and (atp_setup.runall)) then begin
					dataframe = dataframe(*,in_bands)
				endif
				; Use write_bsq_slice to write ns x nb directly into a BSQ file:
				stat = write_bsq_slice(out_lun, atp_setup.nl, j, dataframe)
				if (stat ne 0) then begin
					if (~keyword_set(batch)) then begin
						msg = 'Error writing output; write_bsq_slice code ' + strtrim(stat,2) $
				    	    + string([10b]) + 'Atmospheric correction failed.'
						ok = dialog_message(msg, title='Abort correction')
					endif
					stat = get_cdr4_from_collection(/kill)    ; clear cdrs off the heap 
					if (atm_in_tmp ne '') then begin
						if (n_elements(atm_in_id) gt 0) then begin
							envi_file_mng, id=atm_in_id, /remove, /delete
						endif
						file_delete, atm_in_tmp+'.hdr', /quiet
					endif
					if (outfiletmp1 ne '') then file_delete,outfiletmp1,/quiet
					if (outfiletmp2 ne '') then file_delete,outfiletmp2,/quiet
					return,-1
				endif
			endelse

			; count valid pixels at current line
			k = where((dataframe[*,24] lt 65534.0), ngood)
			ntot += ngood

		endfor    ; end 'nl' loop over lines

		if (atp_setup.enable_artifact) then begin
			f = 100.0 * float(npatch) / ntot
			print, 'Artifact patch applied at '+strtrim(npatch,2) + ' ('+string(f,format='(F5.1)') + '%) pixels.'
		endif

		;--------------------
		;   DONE - CLEAN UP
		;--------------------

		if (n_elements(out_lun) gt 0) then free_lun, out_lun

		stat = get_cdr4_from_collection(/kill)    ; clear the cdrs on the heap 

		if (~keyword_set(batch)) then envi_report_init, base = status_base, /finish

		; Delete input temp file and header if present
		if (atm_in_tmp ne '') then begin
			if (n_elements(atm_in_id) gt 0) then begin
				envi_file_mng, id=atm_in_id, /remove, /delete
			endif
			file_delete, atm_in_tmp+'.hdr', /quiet
		endif
		if (outfiletmp1 ne '') then file_delete,outfiletmp1,/quiet
		if (outfiletmp2 ne '') then file_delete,outfiletmp2,/quiet

	endif else begin     ; end of atm='scale' option
		if (n_elements(out_lun) gt 0) then free_lun, out_lun
		if (~keyword_set(batch)) then begin
			msg = 'No atmospheric correction other than "scale" available.'
			ok = dialog_message(msg, title='Abort correction')
		endif
		if (atm_in_tmp ne '') then begin
			if (n_elements(atm_in_id) gt 0) then begin
				envi_file_mng, id=atm_in_id, /remove, /delete
			endif
			file_delete, atm_in_tmp+'.hdr', /quiet
		endif
		if (outfiletmp1 ne '') then file_delete,outfiletmp1,/quiet
		if (outfiletmp2 ne '') then file_delete,outfiletmp2,/quiet
		return,-1
	endelse
endif



;============================
;    HEADER PREP
;============================

def_bands = atp_setup.def_bands
if ((n_elements(def_bands) eq 3) && (min(def_bands) gt 0)) then begin
	; For header... see note in write_cat_header prolog about 
	; decrementing def_bands
	def_bands -= 1
endif else begin
	def_bands = [0,0,0]
endelse

; Append corrections to history:
cat_history = atp_setup.cat_history
if (strlowcase(atp_setup.phot) eq 'on') then begin
	; Only cosine corection enabled for now:
	cat_history = cat_append_history(cat_history, 'PHCc')
endif
if (strlowcase(atp_setup.therm) ne 'none') then begin
	; Place holder awaiting implementation of thermal correction
endif
if (strlowcase(atp_setup.atm) eq 'scale') then begin
	; For now, has to be volcano scan version 1 (ATv1) with wavelengths either...
	if (npatch eq 0) then begin
		atcode = 'ATCv1'
	endif else begin
		atcode = 'ATCv2'
	endelse
	if (atp_setup.bandset_id eq 0) then begin
		atcode += 'm'
	endif else if (atp_setup.bandset_id eq 1) then begin
		atcode += 'p'
	endif else if (atp_setup.bandset_id eq 2) then begin
		atcode += 'n'
	endif
	; And using volcano scan...
	if (atp_setup.atmt_source eq 'auto') then begin
		atcode += 'AUTO'
	endif else if (atp_setup.atmt_source eq 'tbench') then begin
		atcode += 'SHIFT'
	endif else begin
		atcode += strtrim(atp_setup.vsid_slct,2)
	endelse
	cat_history = cat_append_history(cat_history, atcode)		
endif

; Update input file list:
cat_input_files = atp_setup.cat_input_files
if (n_elements(cat_input_files) gt 0) then cat_input_files = strtrim(cat_input_files,2)
if (  (n_elements(cat_input_files) gt 0) && (cat_input_files[0] ne '-1') $
   && (cat_input_files[0] ne '')) then begin
	cat_input_files = [cat_input_files, file_basename(atp_setup.datafile)]
endif else begin
	cat_input_files = [file_basename(atp_setup.datafile)]
endelse
	
; Spectral subsetting...
if (atp_setup.in_wavesall[0] ne -1) then begin
	wl = atp_setup.in_wavesall[atp_setup.in_bands]
endif
if (atp_setup.in_bbl[0] ne -1) then begin
	bbl = atp_setup.in_bbl[atp_setup.in_bands]
endif
if (atp_setup.fwhm[0] ne -1) then begin
	fwhm = atp_setup.fwhm[atp_setup.in_bands]
endif
if ( (n_elements(def_bands) eq 3) && $
     (min(def_bands) gt 0) ) then begin
	def_bands = crism_def_bands(atp_setup.cat_crism_detector_id, wl)
endif



if (atp_setup.memory) then begin
	print,'SAVE TO MEMORY'
	; If user only wanted a subset of bands but all were run, subset now:
	if ((atp_setup.subset) and (atp_setup.runall)) then begin
		memdata = memdata[*,*,atp_setup.in_bands]
	endif
	envi_enter_data,memdata,wavelength_unit=atp_setup.wu, wl=wl, $
			bbl=bbl, fwhm=fwhm, data_ignore_value=65535.0, $
			def_bands=def_bands, descrip='CAT CRISM DATA', $
			map_info=atp_setup.map_info
endif else begin

	stat = write_cat_header(FNAME=atp_setup.outfile, $
	          NS=atp_setup.ns, NL=atp_setup.nl, $
	          NB=n_elements(atp_setup.in_bands), $                
	          DATA_TYPE=4, OFFSET=0, INTERLEAVE=0, $  ; always for CRISM tile
	          BBL=bbl, DATA_IGNORE_VALUE=65535.0, $
	          DEF_BANDS=def_bands, DESCRIP='CAT CRISM DATA', $
	          WAVELENGTH_UNIT=atp_setup.wu, WL=wl, $
	          FWHM=fwhm, MAP_INFO=atp_setup.map_info, $
	          CAT_PRODUCT_VERSION     = atp_setup.cat_product_version, $
	          CAT_CRISM_DETECTOR_ID = 'J', $
	          CAT_PDS_LABEL_FILE      = atp_setup.lblfile, $
	          CAT_SPECTRUM_RESAMPLED  = atp_setup.resamp, $
	          CAT_IR_WAVES_REVERSED   = atp_setup.cat_ir_waves_reversed, $
	          CAT_EMPIRICAL_GEOM_NORM_FLAG =atp_setup.cat_empirical_geom_norm_flag, $
	          CAT_PHOTOMETRIC_CORR_FLAG =atp_setup.cat_photometric_corr_flag, $
	          CAT_ATMOSPHERIC_CORR_FLAG =atp_setup.cat_atmospheric_corr_flag, $
	          CAT_RATIO_SHIFT_CORR_FLAG =atp_setup.cat_ratio_shift_corr_flag, $
	          CAT_EMPIRICAL_SMILE_CORR_FLAG =atp_setup.cat_empirical_smile_corr_flag, $
	          CAT_SENSOR_SPACE_TRANSFORM_FLAG =atp_setup.cat_sensor_space_transform_flag, $
	          CAT_HISTORY             = cat_history, $
	          CAT_INPUT_FILES         = cat_input_files, $
	          BATCH=batch )

	; Report problems with header if any indicated by status returned:
	if ((stat ne 0) and (~keyword_set(batch))) then begin
		if (stat eq -1) then begin
			msg = 'Problem writing header for '+file_basename(atp_setup.datafile) + string(10b) $
			      + 'Required keywords for write_cat_header undefined.'
			ok = dialog_message(msg, title='Header Error')
		endif else if (stat eq -2) then begin
			msg = 'Problem writing header for '+file_basename(atp_setup.datafile) + string(10b) $
			      + 'Header already exists.'
			ok = dialog_message(msg, title='Header Error')
		endif else begin
			msg = 'Problem writing header for '+file_basename(atp_setup.datafile) + string(10b) $
			      + 'Unspecified error.'
			ok = dialog_message(msg, title='Header Error')
		endelse
	endif

endelse

end
