;******************************************************************************
;
; function catuser_file_search, dialog_msg, search_title, file_filt, user_cancel
;
; Have user search interactively for a file.  (Typically after CAT failed to 
; find it on its own.)
;
; INPUT:
;	dialog_msg    string    A string to display in a dialog before calling 
;	                        envi_pickfile, tell user what to look for.
;	search_title  string    String to display as title of envi_pickfile widget
;	file_filt     string    File search filter (eg 'MRRWV*.TAB' for tile wave table)
;
; OUTPUT: No parameters
;	user_cancel    int     Set to 1 if user cancels; 0 if not.
;
; RETURN VALUE: Selected file.
;
; KEYWORDS: None.
;
; NOTES:
;	Dialog offers user option to cancel; if user canels, return '' and set user_cancel flag.
;
;	If user selects more than one file, returns a string array with all selected.
;
; TYPICAL CALL:
;	dialog_msg = 'CAT failed to find WA CDR. Do you want to search for it?'
;	sttl = 'Find CDR'
;	file_filt = '*WA*.IMG'
;	fnm = catuser_file_search(dialog_msg, sttl, file_filt, user_cancel)
;	if (user_cancel) then retall
;
; USES:
;	pick_latest_pds_version
;
; HISTORY:
;	2009 Jan 03 F. Morgan: First version written
;	2009 Jan 28 FM: Minor documentation update
;	2009 Jul 21 FM: Documentation update - reflecting actual return value when canceled.
;
;******************************************************************************


function catuser_file_search, dialog_msg, search_title, file_filt, user_cancel

forward_function pick_latest_pds_version, envi_pickfile

user_cancel = 0

PICKCATFILE:

; Only put up dialog if msg is a defined, non-empty string:
if (size(dialog_msg,/type) eq 7) then begin
	if (dialog_msg ne '') then begin
		ok = dialog_message(dialog_msg, title='CAT User File Search', /cancel,/question)
		if ((ok eq 'No') or (ok eq 'Cancel')) then begin
			user_cancel = 1
			return,''
		endif
	endif
endif

if (size(search_title,/type) eq 7) then begin
	srchttl = search_title
endif else begin
	srchttl = 'Find CAT file'
endelse
catfile = envi_pickfile(title=srchttl, filter=file_filt)
if (catfile[0] eq '') then begin
	user_cancel = 1
	return,''
endif

; Double-check for file:
stat = cat_file_check(catfile, found_catfile)
if (stat eq 0) then begin
	goto,PICKCATFILE   ; try again
endif else begin
	; If more than one, use the highest version number
	catfile = pick_latest_pds_version(found_catfile)
	if (catfile eq '') then catfile = found_catfile[0]
endelse

return, catfile
end
