;******************************************************************************
;
; function crism_photom_corr, img, inc_ang, img_corr
;
; Photometric correction to a CRISM image slice.
;
; INPUT:
;	img       float[arr[n1,[n2,[n3]]]]   Image slice, optionally 1 or 2 spatial 
;	                                     dimensions, optionally bands as well
;	inc_ang   float[arr[n1,[n2]]         Incidence angles, radians; optionally 
;	                                     1 or 2 spatial dimensions, matching img.
;
; OUTPUT:
;	Parameters:
;		img_corr   flt[arr]   Corrected image same dimension as input IMG
;	Return value:
;		status flag:  0: success
;		             -1: input dimensions invalid
;
; KEYWORDS: None.
;
; NOTES:
;	Inputs can be arrays. 
;
;	Dimensions of INC_ANG must match dimensions of IMG, with the optional 
;	exception that IMG can have one extra dimension that is taken to 
;	correspond to spectral bands. The extra dimension in IMG, if present, must be 
;	the outer-most (last subscript).
;		For example:
;			640 spatial elements, each with potentially different incidence angles,
;			plus 438 spectral bands (like IR channel FRT):
;				img = fltarr(640,438),  inc_ang = fltarr(640)
;			640 spatial elements from a single band:
;				img = fltarr(640),  inc_ang = fltarr(640)
;			Tile plane, single band:
;				img = fltarr(1279,1280), inc_ang = fltarr(1279,1280)
;			A single band from a single pixel:
;				img = 0.3,  inc_ang = 0.097
;			One spectrum from a single pixel in a MSP:
;				img = fltarr(72),  inc_ang = 0.10
;               or... img = fltarr(1,72),   inc_ang = fltarr(1)
;	            [but img = fltarr(72), inc_ang = [0.0] is a problem]
;	            [img=fltarr(1,72), inc_ang=0.0 is an OK special case]
;
;	Bad pixel flag 65535 respected. Carries through to img_corr from any 
;	corresponding element of img. If any element of inc_ang is invalid, 
;	carries through to all corresponding elements of img_corr.
;
; USES:
;
; HISTORY:
;	2008 Nov 18 F. Morgan    First version; tested Nov. 19
;
;******************************************************************************

function crism_photom_corr, img, inc_ang, img_corr

; Figure out dimensions
s_img = size(img)
s_ang = size(inc_ang)

; IMG might have up to 1 more dimensions than INC_ANG. Scalar angle is a 
; special case because might have 0 INC_ANG dimensions and 2 IMG dimensions 
; and still be ok, if img = fltarr(1,n). Make a working copy of inc_ang 
; as a vector in case that happens.
ang = inc_ang
if ((s_ang[0] eq 0) && (s_img[0] eq 2)) then begin
	if (s_img[1] ne 1) then begin
		print,'CRISM_PHOTOM_CORR:  dim(INC_ANG)=0, dim(IMG)=2, first IMG dim != 1'
		return, -1
	endif else begin
		ang = [inc_ang]
		s_ang = size(inc_ang)        ; reset s_ang
	endelse
endif
; Now ANG is a copy of inc_ang possibly array-ized to conform to IMG.

; General case checks (will be OK in the special case now too)
if ((s_img[0] lt s_ang[0]) || (s_img[0] gt s_ang[0]+1)) then begin
	print,'CRISM_PHOTOM_CORR:  Dimensions(INC_ANG) <= Dimensions(IMG) <= Dimensions(INC_ANG)+1'
	return, -1
endif
for i=0,s_ang[0]-1 do begin
	if (s_img[i+1] ne s_ang[i+1]) then begin
		print,'CRISM_PHOTOM_CORR: IMG must match corresponding INC_ANG dimensions'
		return,-1
	endif
endfor

; OK - dimensions passed the tests now.


; ==========================================
;      APPLY CORRECTION
; ==========================================

; Number of elemenets in the extra dimension in IMG (1 if no extra dimension)
n_extra = n_elements(img) / n_elements(inc_ang)
; Reform img to 2D array
img = reform(img, s_ang[s_ang[0]+2], n_extra)
; 1/cosine of angle
ca = cos( reform(ang, s_ang[s_ang[0]+2]) )          ; turn angle matrix into vector
bad_ang = where(((ca eq 0.) or (ang gt 65000.)), nbadang)
if (nbadang gt 0) then ca[bad_ang] = 1.
ca = 1./ca         ; inverse cosine of the angle, except at invalid points
img_corr = img      ; to make matrix
for i=0,n_extra-1 do begin
	img_corr[*,i] *= ca
	if (nbadang gt 0) then img_corr[bad_ang,i] = 65535.
endfor

; Restore invalid values from input image: 
k = where((abs(img-65535.) lt 0.1), nk)
if (nk gt 0) then img_corr[k] = 65535.

; Reform back to original structure:
img = reform(img, s_img[1:s_img[0]])
img_corr = reform(img_corr, s_img[1:s_img[0]])

return,0
end
