;******************************************************************************
;
; function read_crism_ddr, ddr_fnm, ddr_data
;
; ENVI code to read data from a CRISM DDR.
;
; INPUT:
;	ddr_fnm_in	Filename and path for a CRISM DDR file.
;
; OUTPUT PARAMETERS:
;	ddr_data	Structure with ddr data.
;
; RETURN VALUE:
;	Error status:
;		0		Success.
;		-1 		DDR file does not exist.
;		-2		DDR LBL not found
;		-3		Some data in DDR not loaded into output because bandname from LBL
;		  		doesn't match expected structure entries.
;		-4		All DDR arrays are SPARE
;
; KEYWORDS:
;	Keywords specify which parameters are read from the ddr.
;	/allparam reads all parameters and other keywords are disregarded. 
;	If /allparam is not set and other keywords are set, only those keywords are 
;	populated in the output ddr structure.
;	/allparam is the default (if no keywords set).
;
;	Some parameters present in tile MRRDE's are not returned with the current 
;	version of this routine.
;
; EXAMPLE:
;
; NOTES:
; 	DDR Structure:
;		ddr.lat				Latitude, areocentric, deg N
;		ddr.lon				Longitude, areocentric, deg E
;		ddr.ina_areoid		INA at areoid, deg
;		ddr.ema_areoid		EMA at areoid, deg
;		ddr.phase			Phase angle, deg
;		ddr.ina_surface		INA at surface, deg
;		ddr.ema_surface		EMA at surface, deg
;		ddr.slope_mag		Slope magnitude from MOLA, deg
;		ddr.slope_az		MOLA slope azimuth, deg clkwise from N
;		ddr.elev			Elevation, meters relative to MOLA
;		ddr.thermal_inert	Thermal inertia, J m^-2 K^-1 s^-0.5
;		ddr.albedo			Bolometic albedo
;		ddr.lst				Local solar time, hours
;
;		Each tag at that level is actually a structure with two components, e.g.:
;			ddr.lat.loaded = 0 or 1, whether its not loaded, or was loaded by this routine
;			ddr.lat.data = the fltarr output containing latitude data
;
; USES:
;	read_crism_lbl
;
; HISTORY:
;	2008 Apr 04  MFM  First version written.
;	2008 Apr 27  MFM  Made insensitive to filename case for PDS.
;	2009 Jul 09  MFM  Explicitly check cat_file_check return against 0.
;
;******************************************************************************

function read_crism_ddr, ddr_fnm_in, ddr, allparam=allparam, $
		latitude=latitude, longitude=longitude, $
		ina_areoid=ina_areoid, ema_areoid=ema_areoid, phase=phase, $
		ina_surface=ina_surface, ema_surface=ema_surface, $
		slope_mag=slope_mag, slope_az=slope_az, elevation=elevation, $
		thermal_inert=thermal_inert, albedo=albedo, lst=lst, $
		solar_longitude=solar_longitude



; If no specific keywords set, then /allparam
if (~keyword_set(allparam)) then begin
	if ((~keyword_set(latitude)) $
			and (~keyword_set(longitude)) $
			and (~keyword_set(ina_areoid)) $
			and (~keyword_set(ema_areoid)) $
			and (~keyword_set(phase)) $
			and (~keyword_set(ina_surface)) $
			and (~keyword_set(ema_surface)) $
			and (~keyword_set(slope_mag)) $
			and (~keyword_set(slope_az)) $
			and (~keyword_set(elevation)) $
			and (~keyword_set(thermal_inert)) $
			and (~keyword_set(albedo)) $
			and (~keyword_set(lst)) $
			and (~keyword_set(solar_longitude))) then begin
		allparam = 1
	endif
endif


; Make sure input file exists. Accept any case.
if (cat_file_check(ddr_fnm_in, found_ddr) eq 0) then return,-1
ddr_fnm = found_ddr[0]

; Find the LBL for the DDR:
p = strpos(ddr_fnm, '.', strlen(ddr_fnm)-1, /reverse_search)
ddr_lbl_fnm = strmid(ddr_fnm,0,p) + '.LBL'

if (cat_file_check(ddr_lbl_fnm, lblfound) eq 0) then return, -2
ddr_lbl_fnm = lblfound[0]

; Find image dimensions, band names, etc from LBL
read_crism_lbl, ddr_lbl_fnm, $
	['LINES', 'LINE_SAMPLES', 'BANDS', 'BAND_STORAGE_TYPE', 'BAND_NAME'], kwv
nlines = long(kwv[0])
nsamp = long(kwv[1])
nbands = long(kwv[2])
bst = kwv[3]              ; band storage type
band_name_list = kwv[4]

; Make an array of the dimensions of DDR data:
if (strpos(strupcase(bst),'BAND_SEQUENTIAL') ge 0) then begin
	img = fltarr(nsamp, nlines)
endif else begin
	img = fltarr(nlines, nsamp)
endelse
img[*,*] = 65535.


; Output structure shell:
ddr = { lat: {loaded: 0, data: img}, $
		lon: {loaded: 0, data: img}, $
		ina_areoid: {loaded: 0, data: img}, $
		ema_areoid: {loaded: 0, data: img}, $
		phase: {loaded: 0, data: img},  $
		ina_surface: {loaded: 0, data: img}, $
		ema_surface: {loaded: 0, data: img}, $
		slope_mag: {loaded: 0, data: img}, $
		slope_az: {loaded: 0, data: img}, $
		elev: {loaded: 0, data: img}, $
		thermal_inert: {loaded: 0, data: img}, $
		albedo: {loaded: 0, data: img}, $
		lst: {loaded: 0, data: img}, $
		solar_longitude: {loaded: 0, data: img} $
		}


openr, uddr, ddr_fnm, /SWAP_IF_BIG_ENDIAN, /get_lun

; Fill in available data
xx = strupcase(strsplit(band_name_list, '"' ,/extract))
odd_idx = 2 * indgen(n_elements(xx)/2) + 1
bname_array = xx[odd_idx]
; drop any spares...
k = where((strpos(bname_array,'SPARE') lt 0), nk)
if (nk eq 0) then begin
	free_lun, uddr
	return, -4 
endif
bname_array = bname_array[k]

for i=0,n_elements(bname_array)-1 do begin
	readu, uddr, img
	if (strpos(bname_array[i], 'INA AT AREOID') ge 0) then begin
		if (keyword_set(allparam) or keyword_set(ina_areoid)) then begin
			ddr.ina_areoid.loaded = 1
			ddr.ina_areoid.data = img
		endif
	endif else if (strpos(bname_array[i], 'EMA AT AREOID') ge 0) then begin
		if (keyword_set(allparam) or keyword_set(ema_areoid)) then begin
			ddr.ema_areoid.loaded = 1
			ddr.ema_areoid.data = img
		endif
	endif else if (strpos(bname_array[i], 'PHASE ANGLE') ge 0) then begin
		if (keyword_set(allparam) or keyword_set(phase)) then begin
			ddr.phase.loaded = 1
			ddr.phase.data = img
		endif
	endif else if (strpos(bname_array[i], 'INA AT SURFACE') ge 0) then begin
		if (keyword_set(allparam) or keyword_set(ina_surface)) then begin
			ddr.ina_surface.loaded = 1
			ddr.ina_surface.data = img
		endif
	endif else if (strpos(bname_array[i], 'EMA AT SURFACE') ge 0) then begin
		if (keyword_set(allparam) or keyword_set(ema_surface)) then begin
			ddr.ema_surface.loaded = 1
			ddr.ema_surface.data = img
		endif
	endif else if (strpos(bname_array[i], 'SLOPE MAGNITUDE') ge 0) then begin
		if (keyword_set(allparam) or keyword_set(slope_mag)) then begin
			ddr.slope_mag.loaded = 1
			ddr.slope_mag.data = img
		endif
	endif else if (strpos(bname_array[i], 'SLOPE AZIMUTH') ge 0) then begin
		if (keyword_set(allparam) or keyword_set(slope_az)) then begin
			ddr.slope_az.loaded = 1
			ddr.slope_az.data = img
		endif
	endif else if (strpos(bname_array[i], 'ELEVATION') ge 0) then begin
		if (keyword_set(allparam) or keyword_set(elevation)) then begin
			ddr.elev.loaded = 1			
			ddr.elev.data = img
		endif
	endif else if (strpos(bname_array[i], 'SOLAR LONGITUDE') ge 0) then begin
		; NOTE:::  THIS ***HAS*** TO BE BEFORE PLAIN "LONGITUDE"
		if (keyword_set(allparam) or keyword_set(solar_longitude)) then begin
			ddr.solar_longitude.loaded = 1
			ddr.solar_longitude.data = img
		endif
	endif else if (strpos(bname_array[i], 'LATITUDE') ge 0) then begin
		if (keyword_set(allparam) or keyword_set(latitude)) then begin
			ddr.lat.loaded = 1
			ddr.lat.data = img
		endif
	endif else if (strpos(bname_array[i], 'LONGITUDE') ge 0) then begin
		if (keyword_set(allparam) or keyword_set(longitude)) then begin
			ddr.lon.loaded = 1
			ddr.lon.data = img
		endif
	endif else if (strpos(bname_array[i], 'THERMAL INERTIA') ge 0) then begin
		if (keyword_set(allparam) or keyword_set(thermal_inert)) then begin
			ddr.thermal_inert.loaded = 1
			ddr.thermal_inert.data = img
		endif
	endif else if (strpos(bname_array[i], 'ALBEDO') ge 0) then begin
		if (keyword_set(allparam) or keyword_set(albedo)) then begin
			ddr.albedo.loaded = 1
			ddr.albedo.data = img
		endif
	endif else if (strpos(bname_array[i], 'LOCAL SOLAR TIME') ge 0) then begin
		if (keyword_set(allparam) or keyword_set(lst)) then begin
			ddr.lst.loaded = 1
			ddr.lst.data = img
		endif
	endif
endfor

free_lun, uddr
return, 0
 
end
