;Procedure to search for CRISM label file for a 
; datafile name with the full path.
;Makes sure detectors and filenames match.
;
;Input datafile (filename with full path).
;
;Returns label filename with full path.
;Returns lblfile='' if lblfile not found.
;Returns lblfile='' if cancel selected.
;
; Precedence:
;	1) Label file specified in CAT header
;	2) Same filename with .LBL extension, in same directory
;	3) Cleaned filename with .LBL extension, in same directory
;	4) Cleaned filename with .LBL extension, in PDS directory
;	5) User search
;	(Cleaned filename means stripping any CAT-appended text between 
;	the PDS root and extension; e.g. FRT000028BA_07_IF165L_TRR1_corr.img
;	becomes FRT000028BA_07_IF165L_TRR1.img.)
;
; USES:
;	cat_file_check
;	cat_envi_open_ifnot
;	read_cat_header
;	catuser_file_search
;
; HISTORY:
;NOV 03, 2006:  Created for CAT processing by S.Pelkey
;
;Modification History:
; FEB 16,2007 SMP: --Modified to create a .sav file
; APR 24,2008 MFM: --Made case-insensitive to handle PDS lower-case names
;                    or CRISM upper-case names
; Feb 10,2009 MFM: --For checking file types from filename contents, 
;                    use only file basename, don't include path.
; Jun 10,2009 MFM: --Search PDS tree for label if not found in same directory 
;                    as datafile_in
; Jun 15,2009 MFM: --use cat_pds_path instead of looking up PDS root directly 
;                    in config file
; Jun 15,2009 MFM: --call reconstruct_pds_path for PDS search
; Jul 20,2009 MFM: --total revamp. clarify precedence of different 
;                    candidate labels. Tested.
; Sep 16,2009 MFM: --added BATCH keyword - no interactive search if set
; Sep 17,2009 MFM: --find_lblfile returns '' not '-1' on fail
; Nov 12,2009 MFM: --No longer search explicitly for header file. ENVI_open the 
;                    image file and use read_cat_header to get label filename 
;                    if present. Always return 'n/a' for hdrfile.
; Nov 26,2009 MFM: --Add status notification.
; Jan 29,2010 MFM: --Add lower case file filter.
; Apr 13,2012 MFM: --Skip ENVI header search in BATCH mode to avoid IDL-only fail
; Oct 08,2015 MFM: --Add invisible keyword on cat_envi_open_ifnot to avoid extra dialog

pro find_lblfile, datafile_in, lblfile, hdrfile, batch=batch


forward_function cat_file_check, cat_envi_open_ifnot, read_cat_header, catuser_file_search


;=====================================
;   NOTIFY SEARCHING FOR LABEL
;=====================================

if (~keyword_set(batch)) then begin
	status_string_array = ['Searching for PDS label for', $
                       file_basename(datafile_in)]
	envi_report_init, status_string_array, base = status_base, title = ''
endif


;;;;;;;;;;;;;;;
; SET DEFAULTS:
;;;;;;;;;;;;;;;
lblfile = ''
hdrfile = 'n/a'
filter = ''

; Important to trim blanks... won't find ' filename ' 
datafile = strtrim(datafile_in,2)


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; 1) Look for a label sepcfied in a CAT header file
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

; Can't use envi_get_file_ids outside ENVI, so don't try this in batch mode
if (~keyword_set(batch)) then begin
	cat_envi_open_ifnot, datafile, r_fid, already_open, /no_realize, $
	     /no_interactive_query, /invisible
	if (r_fid ge 0) then begin
		stat = read_cat_header(r_fid, CAT_PDS_LABEL_FILE=try_lblfile)
		if (~already_open) then envi_file_mng, id=r_fid, /remove
		lbl_fnd = cat_file_check(try_lblfile, found_lbl)
		if (lbl_fnd gt 0) then begin
			; Exists, return it. If it didn't, then leave lblfile=''.
			lblfile = found_lbl[0]
			if (~keyword_set(batch)) then envi_report_init, base=status_base, /finish
			return
		endif
	endif
endif
; If label found, returned.
; If didn't find a header file, then still hdrfile = 'n/a'


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; 2) Look for .LBL on same name as datafile, in same directory
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

try_lblfile = datafile
p = strpos(try_lblfile, '.', /reverse_search)
if (p gt 0) then begin
	strput, try_lblfile, '.LBL', p
	lbl_fnd = cat_file_check(try_lblfile, found_lbl)
	if (lbl_fnd gt 0) then begin
		; Found a matching label file
		lblfile = found_lbl[0]
		if (~keyword_set(batch)) then envi_report_init, base=status_base, /finish
		return
	endif
endif
; If label found, returned.
; If didn't find a label, then still lblfile = ''


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; 3) Look for .LBL on cleaned datafile name, in same directory
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

try_lblfile = strip_catfile_appendage(datafile)
p = strpos(try_lblfile, '.', /reverse_search)
if (p gt 0) then begin
	strput, try_lblfile, '.LBL', p
	lbl_fnd = cat_file_check(try_lblfile, found_lbl)
	if (lbl_fnd gt 0) then begin
		; Found a matching label file
		lblfile = found_lbl[0]
		if (~keyword_set(batch)) then envi_report_init, base=status_base, /finish
		return
	endif
endif
; If label found, returned.
; If didn't find a label, then still lblfile = ''


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; 4) Look for .LBL on cleaned datafile name in PDS directory tree
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

data_basename = strip_catfile_appendage( file_basename(datafile))
pds_path = reconstruct_pds_path(data_basename)
if (pds_path ne '') then begin
	p = strpos(data_basename, '.', /reverse_search)
	if (p gt 0) then begin
		strput, data_basename, '.LBL', p
		try_lblfile = pds_path + data_basename
		lbl_fnd = cat_file_check(try_lblfile, found_lbl)
		if (lbl_fnd gt 0) then begin
			; Found a matching label file
			lblfile = found_lbl[0]
			if (~keyword_set(batch)) then envi_report_init, base=status_base, /finish
			return
		endif
	endif
endif
; If label found, returned.
; If didn't find a label, then still lblfile = ''


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; 5) Still no label; let user search if they wish
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

if (~keyword_set(batch)) then begin
	msg = 'LBL file not found for '+strtrim(datafile,2) + string(10B) $
		+ 'Would you like to locate it?'
	srch_ttl = 'Choose LBL file'
	file_filt = '*.lbl;*.LBL'

	user_lbl = catuser_file_search(msg, srch_ttl, file_filt, user_cancel)

	if (user_cancel) then begin
		lblfile = ''
	endif else begin
		if (file_test(user_lbl)) then lblfile = user_lbl
	endelse
endif else begin
	lblfile = ''
endelse

if (~keyword_set(batch)) then envi_report_init, base=status_base, /finish
return
end
