;******************************************************************************
;
; function get_catcfg_param
;
; Get values for one or more CAT config parameters.
;
; INPUT:
;	param_name      string or strarr      Name(s) of CAT config parameters to get
;
; OUTPUT:
;	Parameters: none
;	Return value: parameter value(s)
;
; KEYWORDS: none
;
; NOTES:
;	Calls load_cat_config to load config file params into struct in 
;	common block CRISMCAT_CFGPARAMS if the struct is not defined already. 
;
;	If input param_name is scalar, returns a scalar parameter. 
;	If inpt is an array, returns a matching strarr containing 
;	parameter values. 
;	Empty string is returned for any parameter names not present in the 
;	config file structure.
;
;	Parameter names are not case sensitive.
;
; USES:
;	load_cat_config
;
; HISTORY:
;	2008 Sep 1 F. Morgan: First version written and tested.
;
;******************************************************************************

function get_catcfg_param, param_name

common CRISMCAT_CFGPARAMS, cfgparam


; Verify arguments and config file available:
if (n_elements(param_name) eq 0) then return,''
if (n_elements(cfgparam) eq 0) then load_cat_config 
; Fail if config file not present:
if (~fix(cfgparam.cfgpresent)) then begin
	nocfg = param_name
	nocfg[*] = ''
	return, nocfg
endif

cfg_tags = strupcase(strtrim(tag_names(cfgparam),2))

nreq = n_elements(param_name)
param_val = strarr(nreq)
for i=0,nreq-1 do begin
	pnm = strupcase(strtrim(param_name[i],2))

	; Look for a matching tag in the config structure
	k = where((cfg_tags eq pnm), nk)
	if (nk eq 0) then begin
		param_val[i] = ''     ; Not present
	endif else begin
		; Parameter is present in structure. Pull the value...
		param_val[i] = cfgparam.(k[0])
	endelse
endfor
; If input scalar, return scalar:
if (size(param_name, /n_dimensions) eq 0 ) then param_val=param_val[0]

return, param_val
end
