;******************************************************************************
;
; function get_vs_artifact, vscdr, scale_waves, sw, interp_waves, batch=batch
;
; Get a volcano scan artifact spectrum from a VS ADR or AT CDR.
;
; INPUT:
;	vscdr        string      CDR filename (VS ADR or AT CDR)
;	scale_waves  fltarr(2)   Scaling wavelengths
;	sw           fltarr(480) SW CDR wavelengths
;
; OUTPUT PARAMETERS:
;	interp_waves  fltarr(2)  Interpolation wavelengths listed in the CDR
;
; RETURN VALUE:
;	Selected artifact; -1 if file not found or matching artifact not in file
;
; KEYWORDS:
;	BATCH     If set, no interactive searching.
;
; NOTES:
;	Looks in CDR label comments for line like 
;   /* Line 3: [2011.0,1899.0], Pelkey ...
;	to determine scaling wavelengths of the artifacts stored in the CDR
;	(if there are any). There's some slop in wavelengths that still 
;	corresponds to nearly the same scaling... Determines the 
;	corresponding sweetspot bands and compares to the sweetspot bands 
;	of the input scale_waves vector. If the pairs match, that identifies 
;	the CDR line to pull. If none match return -1.
;	First artifact with scaling wavelengths that match bands gets used.
;
; USES:
;	find_lblfile
;	
;
; HISTORY:
;	2012 Mar 15 FM  Created
;	2012 Mar 20 FM  Documentation edit
;	2012 Apr 13 FM  Add SW input; replace search for SW CDR.
;	2012 Apr 13 FM  Fix forward function and update "USES" list
;
;******************************************************************************

function get_vs_artifact, vscdr, scale_waves, sw, interp_waves, batch=batch

forward_function find_lblfile


MAX_ART = 8    ; max number of artifacts

if (~file_test(vscdr)) then return,-1


;============    GET ARTIFACT INFO FROM CDR LABEL COMMENTS   ==========
; Find CDR label...
find_lblfile, vscdr, lbl, hdr, batch=batch
if (lbl eq '') then return,-1

; Collect artifact line description comments:
s = ''

openr,u, lbl, /get_lun
s = ''
art_num = intarr(MAX_ART)
art_wscale = fltarr(2,MAX_ART)
nart = 0
xw = fltarr(2)
while (~eof(u)) do begin
	readf,u,s
	s = strtrim(s,2)
	if (stregex(s,'^/\* *Line [2-9]: *\[',/boolean)) then begin
		; this is a artifact description line like 
		; /* <whitespace> Line N: <whitespace> [ wavelengths,..., N>= 2
		p1 = strpos(s,'Line') + 4
		p2 = strpos(s,':')
		art_num[nart] = fix(strmid(s,p1,p2-p1))
		p3 = strpos(s,'[')+1
		p4 = strpos(s,']')
		reads, strmid(s,p3,p4-p3), xw
		art_wscale[*,nart] = xw
		nart++
	endif
endwhile
if (nart eq 0) then begin
	if (~keyword_set(batch)) then begin
		msg = 'No artifact data in CDR '+ file_basename(vscdr) + '.'
		ok = dialog_message(msg, title='Artifact selection failed')
	endif
	return, -1
endif
; Now art_num[i] = the label-comment artifact number (running 2,3,4,...)
; and art_wscale[*,i] = scaling wavelengths, for the Ith artifact listed 
; in the label.

; get interpolation wavelengths while we have the label open...
point_lun, u, 0
interp_found = 0
while ((~eof(u)) && (~interp_found)) do begin
	readf,u, s
	if (strpos(s,'interpolation wavelengths were') gt 0) then interp_found=1
endwhile
interp_waves = [-1.0, -1.0]
if (interp_found) then begin
	; The wavelengths themselves are on the next line:
	readf,u, s
	p = strpos(s,'/*')
	s = strmid(s,p+2)
	p = strpos(s,'nm')
	reads, strmid(s,0,p), interp_waves
endif

; done with the label now-
free_lun,u


;==============   SELECT AN ARTIFACT    ==============

; Identify bands for the requested scaling wavelengths and for the 
; artifact scaling wavelengths
req_bnd = intarr(2)
art_bnd = intarr(2,nart)
for i=0,1 do begin     ; loop over scale wave
	dum = min(abs(sw - scale_waves[i]), b)
	req_bnd[i] = b
	for j=0,nart-1 do begin     ; loop over artifacts in CDR
		dum = min(abs(sw - art_wscale[i,j]), b)
		art_bnd[i,j] = b
	endfor
endfor
; sort both in increasing band index order so order of input waves doesn't matter
req_bnd = req_bnd[sort(req_bnd)]
for j=0,nart-1 do art_bnd[*,j] = art_bnd[sort(art_bnd[*,j]), j]

; Select artifact that matches the requested scaling wavelengths
art_idx = -1              ; selected artifact
k = where((((art_bnd[0,*]-req_bnd[0]) eq 0) and ((art_bnd[1,*]-req_bnd[1]) eq 0)), nk)
if (nk gt 0) then begin
	; Assume artifacts are stored in the same order they're listed:
	art_idx = k[0]
endif else begin
	if (~keyword_set(batch)) then begin
		msg = ['No artifact in '+file_basename(vscdr) + ' has scaling wavelengths', $
		       'matching the requested wavelengths ' $
		           +string(scale_waves, format="(F7.1,',',F7.1)")]
		ok = dialog_message(msg, title='Artifact selection failed', /cancel)
	endif
	return, -1
endelse


;==============   LOAD THE ARTIFACT    ==============
read_crism_lbl, lbl, ['LINE_SAMPLES','BANDS','LINES'], val
ns = fix(val[0])
nb = fix(val[1])
nl = fix(val[2])

openr,u, vscdr, /get_lun
aa = assoc(u,fltarr(ns,nb))
idx = art_idx + 1      ; allowing for the image plane
if (idx ge nl) then begin
	if (~keyword_set(batch)) then begin
		msg = 'Selected artifact index exceeds storage in CDR.'
		ok = dialog_message(msg, title='Artifact selection failed', /cancel)
	endif
	return, -1
endif
art = aa[idx]
free_lun,u

return, art
end
