;******************************************************************************
;
; function list_vsat, vsatid, aux_path=aux_path
;
; Generate a list of volcano scan observation IDs.
;
; INPUT: no parameters
;
; OUTPUT:
;	vsatid    strarr    Array of observation IDs of all volcano scans 
;	                    with VS ADR files 
;
; RETURN VALUE:
;	status: 0=success, -1=failed to find any VS AT files
;
; KEYWORDS:
;	aux_path   Path to CAT aux_files; prevents call to cat_auxfile_path() 
;
; NOTES:
;
; USES:
;	cat_auxfile_path
;
; HISTORY:
;	2009 Jan 01   F. Morgan: First version
;	2009 Jan 16   FM: Update for VS ADR filename convention
;	2009 Apr 22   FM: Replace aux = get_catcfg_param('CAT_AUXFILES_PATH') 
;	              with aux = cat_auxfile_path()
;	2009 Aug 28   FM: Update for new VS ADR filenames 
;	2012 Apr 16   FM: Added aux_path keyword
;
;******************************************************************************

function list_vsat, vsatid, aux_path=aux_path

forward_function cat_auxfile_path

; Get aux-file path:

aux = ''
if (size(aux_path,/type) eq 7) then begin
	if (file_test(aux_path[0])) then begin
		aux = (file_search(aux_path[0], /mark_directory))[0]
	endif
endif
if (aux eq '') then begin
	aux = cat_auxfile_path()
endif
if (aux eq '') then return,-1
aux = strtrim(aux,2)
ps = path_sep()
if (strmid(aux,strlen(aux)-1,1) ne ps) then aux += ps

; Path to VSAT ADRs:
vspath = aux + 'ADR' + ps + 'VS' + ps

; List of all files:
filt = vspath + 'ADR*_VS*.IMG'
allvsfiles = file_search(filt,/fold_case)
if (allvsfiles[0] eq '') then return,-1
nfall = n_elements(allvsfiles)
allid = strarr(nfall)
for i=0,nfall-1 do begin
	bn = file_basename(allvsfiles[i])
	x = strsplit(bn,'_',/extract)
	allid[i] = x[1]      ; filenames are ADRPTTTTTTTTTT_DDDDD_... where DDDDD=obs ID
endfor

; There can be multiple files for same observatin, with 
; different binning, etc so sort and pull unique copies only
allid = strupcase(allid)     ; upcase
allid = allid[sort(allid)]   ; sort alphabetically
vsatid = allid[uniq(allid)]  ; Unique IDs only

return,0
end
