;******************************************************************************
;
; function mtrdr_support_fnm, img_fnm, othertype
;
; Given a filename for a MTRDR-related file, make a filename for a different 
; tile file type for the same tile. (e.g., generate the filename for the WV 
; file associated with FRT000094F6_07_IF166J_MTR3.IMG)
;
; INPUT:
;	img_fnm     string     Image tile filename 
; 	othertype   string     Supporting file type ('DE', 'WV', 'IN', 'SU', 'SR')
;
; OUTPUT: no parameters
;
; RETURN VALUE:
;	Supporting file filename, '' if fail.
;
; KEYWRODS:
;	BATCH    If set, do not ask for user search.
;
; NOTES:
;	Image filename is like FRT000094F6_07_IF166J_MTR3.IMG. Might also have some 
;	post-processing tags like FRT000094F6_07_IF166J_MTR3_cat.IMG. 
;
;	This will drop any CAT-appended text in the input filename.
;
; HISTORY:
;	2012 Mar 15   F.Morgan   created
;
;******************************************************************************

function mtrdr_support_fnm, img_fnm, othertype, batch=batch


forward_function find_wavelength_file, reconstruct_pds_path, get_pds_version, $
          catuser_file_search, pdsfile_is_mtrdr

; Fail if not MTRDR:
if (~pdsfile_is_mtrdr(img_fnm, /allow_unproj)) then return,''

otup = strupcase(strtrim(othertype,2))


;===========================================================================
;  SPECIAL CASE: WV FILE
;===========================================================================

; For WV file, look first for the wavelength file called out in label
if (otup eq 'WV') then begin
	tile_fnm = find_wavelength_file(img_fnm)
	if (tile_fnm ne '') then return,tile_fnm
endif
		

;===========================================================================
;  FIND A BASENAME FOR THE SUPPORT FILE
;===========================================================================

; Separate directory and filename:
img_dir = file_dirname(img_fnm,/mark_directory)
img_base = file_basename(img_fnm)

; Substitute to form filter for new filename:
new_fnm = strupcase(img_base)
pos = strpos(new_fnm, '_MTR')
if (pos lt 0) then begin
	pos = strpos(new_fnm, '_TER')
endif
; (One of those must hit if it passed pdsfile_is_mtrdr above)
strput, new_fnm, otup, pos-6    ; specified other type
strput, new_fnm, '?', pos+4     ; any version
if (otup eq 'DE') then begin
	; DE is L, not J:
	strput, new_fnm,'L', pos-1
endif
ext = '.IMG'
if (strpos(otup,'WV') ge 0) then ext = '.TAB'
new_fnm = strmid(new_fnm,0,pos+5) + ext


;=====================================================================
;  FIND A PATH FOR THE SUPPORT FILE
;=====================================================================

; First, try the same directory where the original file was...
try_file = img_dir + new_fnm
fnd_file = file_search(try_file, /fold_case)
if (fnd_file[0] eq '') then begin
	; local path didn't work; try PDS path...
	try_file = reconstruct_pds_path(img_base) + new_fnm
	fnd_file = file_search(try_file, /fold_case)
endif

; Now pick a version if found more than one file:
if (n_elements(fnd_file) gt 1) then begin
	; Try to get a version same as input...
	fnd_vers = get_pds_version(fnd_file, latest_vers)
	img_vers = (get_pds_version(img_base))[0]
	k = where((fnd_vers eq img_vers),nk)
	if (nk gt 0) then begin
		tile_fnm = fnd_file[k[0]]
	endif else begin
		; Nope; use highest version found...
		k = where((fnd_vers eq latest_vers),nk)
		if (nk gt 0) then tile_fnm = fnd_file[k[0]]  ; should always be true
	endelse
endif else begin
	tile_fnm = fnd_file[0]
endelse
if (tile_fnm ne '') then return, tile_fnm

;====================================================================
;  STILL NO LUCK - USER SEARCH
;====================================================================
if (~keyword_set(batch)) then begin
	msg = othertype +' file not found for ' + string(10B) $
	        + strtrim(img_base,2) + string(10B) $
			+ 'Would you like to locate it?'
	srch_ttl = 'Choose ' + othertype + ' file'
	file_filt = '*.*'
	user_fnm = catuser_file_search(msg, srch_ttl, file_filt, user_cancel)
	if (user_cancel) then begin
		tile_fnm = ''
	endif else begin
		if (file_test(user_fnm)) then tile_fnm = user_fnm
	endelse
endif

return, tile_fnm
end
