; NAME:     READ_CRISM_LBL
;
; PURPOSE:
;     The READ_LBL procedure reads CRISM LBL files in PDS format.
;     Parameters are retrieved by specifying the related keywords.
;
; CALLING SEQUENCE:
;     read_crism_lbl, filename,keywords, values
;
; INPUTS:
;    filename: data file name (with path)
;    keywords: string array containing the list of keyword
;
; OUTPUTS:
;    values: string array with same dimensions as the keywords argument;  
;            holds output values as strings.
;            Returns empty string element ('') if file or requested keyword
;            is not found.
;
; NOTES: 
; --program first checks for label file, if not found, '-1' is returned
;
; CREATED: March 2006
;      BY: S. Pelkey
;
; MODIFICATION HISTORY:
; 07-MAR-2006, SMP: Adapted from OMEGA read_geolbl written by N.Manaud.
; 20-JUN-2006, SMP: Modified to handle multi-line "BAND_NAME" in 
;                   *DDR*LBL files
; 25-JUL-2006, SMP: Added ability for user to point to LBL file if not
;                   automatically found.
; 08-AUG-2006, SMP: Made compatible with Windows platform
; 31-OCT-2006, SMP: Added more refined check of platform.
; 20-MAR-2007, SMP: Modified to handle multi-line "BAND_NAME" for any
;                   standard CRISM product that would have them (I think!)
; 27-APR-2008, MFM: Made insensitive to fileame case
; 06-Nov-2008, MFM: Added eof stop on while loop for non-compliant labels without 'END'
; 08-Jan-2009, MFM: Use pick_latest_pds_version() to read the latest label, 
;                   instead of first in the list returned by cat_file_check,
;                   and add file-not-found check
; 20-Jan-2009, MFM: Add MRRDL to list of files requiring PDS band names
; 23-Jan-2009, MFM: Read keywords with multi-line lists demarcated by {...}
; Feb 10,2009 MFM: --For checking file types from filename contents, 
;                    use only file basename, don't include path.
; Jul 09,2009 MFM: --Explicitly check cat_file_check return against 0.
; Jul 16,2009 MFM: --Include LDR (DDR for limb scans) in list of files with band names
; Mar 17,2012 MFM: --Revised bracketed list value search


PRO read_crism_lbl, lblfile_input, keywords, values

forward_function cat_file_check, pick_latest_pds_version

nitems = n_elements(keywords)
values = strarr(nitems)

; Find a file that exists and matches input except possibly case:
if (cat_file_check(lblfile_input, found_lbl) eq 0) then return
lblfile = pick_latest_pds_version(found_lbl)


; If LBL file present, parse to get info...
if (~file_test(lblfile)) then return

; Check if need to deal with band names:
lblup = strupcase(file_basename(lblfile))
pos = strpos(lblup,'DDR') > strpos(lblup,'_LDR') > strpos(lblup,'SU')  $
    > strpos(lblup,'MRRDE')  > strpos(lblup,'MRRDL') > strpos(lblup,'MRRSP') $
    > strpos(lblup,'_DE')
if (pos ne -1) then pdsbands=1 else pdsbands=0

; Open file:
openr, unit, lblfile, /GET_LUN

line    = ''

while ((strtrim(line,2) ne 'END') && (~eof(unit))) DO BEGIN
  ;Read the header one line at a time
  readf, unit, line
  len=strlen(line)
  i = strpos(line,' = ')
  if ( i ne -1 ) then begin
    keyword = strtrim(strmid(line,0,i),2) ;all to the left of =
    value   = strtrim(strmid(line,i+3,len-i-2),2)    ;all to the right of =
    for j=0,nitems-1 do begin
      if ( strupcase(keywords[j]) eq strupcase(keyword) ) then begin
        if (stregex(value,'^[\{\(]') ge 0) then begin
          ; Have the start of a set enclosed by {} or () but maybe not done...
          while (stregex(value,'[\}\)]$') lt 0) do begin
            readf, unit, line
            value += strtrim(line,2)
          endwhile	
          values[j] = value
        endif else begin
          values[j] = value
        endelse
      endif
    endfor
 endif
ENDWHILE

close,    unit
free_lun, unit


END
