;******************************************************************************
;
; function algk_surface_temp_image, lsubs, lat, localhour, albedo, thrm_inert, $
;		slope, slope_az, atmopac, elev
;
; Use AlgK thermal model (AlgK KRC Kieffer Physical Thermal Model) to 
; produce a map of surface temperature corresponding to a CRISM image.
;
;
; INPUT PARAMETERS:
;	lsubs		float		L_s (solar longitude) (degrees)
;	lat			float		Latitude (degrees)
;	localhour	float		Local time (hours, [0,24))
;	albedo		fltarr		Surface hemispherical albedo [0,1]
;	thrm_inert	fltarr		Surface thermal inertia 
;	slope		fltarr		Surface slope (degrees)
;	slope_az	fltarr		Azimuth of surface slope (degrees clockwise from N)
;	atmopac		fltarr		Atmospheric opacity, visible band
;	elev		fltarr		Surface elevation (km)
;
;	(albedo, thrm_inert, slope, slope_az, atmopac, elev are all 2-D arrays with 
;	same dimensions, corresponding to a CRISM image)
;
; OUTPUT PARAMETERS:  none
;
; RETURN VALUE:
;	Surface temperature (K), fltarr of same dimensions as albedo etc. 
;
; KEYWORDS:
;	
;
; NOTES:
;
; USES:
;
; HISTORY:
;	2008 Mar 21 F. Morgan: First version.
;
; ===============================================================
; ===============================================================
;                CREDIT FOR AlgK THERMAL MODEL
;
;  ADR_TE AlgK KRC Kieffer Physical Thermal Model
;
;   Hugh Kieffer, Tim Titus, Patrick McGuire
; ===============================================================
; ===============================================================
;
;
;******************************************************************************

function algk_surface_temp_image, lsubs, lat, localhour, albedo, thrm_inert, $
		slope, slope_az, atmopac, elev



; ====================================
;     SETUP
; ====================================

; Path to ADR
if (strpos(strupcase(!version.os_family),'UNIX') ge 0) then begin
	; unix ADR path
	path_ADR_TE = '~/CAT_ENVI/aux_files/ADR/TE/'
endif else begin
	; windows ADR path
	p = strpos(!DIR,':')
	if (p lt 0) then begin
		drvletr = ''
	endif else begin
		drvletr = strmid(!DIR,0,p+1)
	endelse
	path_ADR_TE = drvletr +'\CAT_ENVI\aux_files\ADR\TE\'
endelse

; ADR filename:
fname = 'ADR_TE_0000_0_0000000000_0L.DAT'

; Turn off noise from bin6r
verbose_bin6r = 0

             ;//////////////////////////////////////////////////////
             ;   COME HERE TO FIX ARCHITECTURE NOT FOUND MESSAGE   :
             ;//////////////////////////////////////////////////////

; Known architectures
; !!!   This is used later to figure out whether to swap bytes in the ADR.
; !!!   Your architecture should be represented in either the "sparc" 
; !!!   family or the "x86" family. If you get a message indicating 
; !!!   "architecture not found" then you need to add yours to the structure.
; !!!   You can run the stand-alone routine byte_order_family.pro
; !!!   in CAT_ENVI/save_add/CAT_programs/Supplementals/thermal_model 
; !!!   which will tell you your architecture string and which byte-swap 
; !!!   family it belongs to.
;
;  types_instruc = { sparc_family: <string array with sparc-like !version.arch values>, $
;                    x86_family: <string array with x86-like !version.arch values> }

types_instruc = { sparc_family: ['sparc'], $
                  x86_family: ['alpha', 'x86  ', 'x86_64   ', 'i386'] }


; ====================================
;     GET SUB-LUT (AT Ls, LAT, LT)
; ====================================

first_call = 0
lut_3,lsubs,localhour,lat,sublut,first_call,path_ADR_TE,fname,types_instruc,verbose_bin6r
	
; ====================================
;     GET TEMPERATURE IMAGE
; ====================================

s = size(albedo)
nx = s[1]
ny = s[2]
temp_img = fltarr(nx,ny)
for j=0,ny-1 do begin
	for i=0,nx-1 do begin
		temp_img[i,j] = fast_3_edit(sublut, albedo[i,j], thrm_inert[i,j], slope[i,j], $
		                  slope_az[i,j], atmopac[i,j], elev[i,j])
	endfor
endfor

return, temp_img
end
