;FAST	Thermal lookup table code for MRO CRISM.
;	This code accesses a subset of the large lookup table
;	that has been preselected for a given Ls, lat, and local time
;	by the routine 'LUT'. This input is called sublut.
;	Preselection is done since a given CRISM image segment can
;	be assumed to have one local time, lat, and Ls value.
;	NB: the correct lat length is TBD pending testing.
;	We use fast interpolation to find the kinetic temperature 
;	appropriate for each pixel within the image segment.
; 

;TZ Martin	July 2005
;       Updated Oct 2005 for actual LUT from USGS (T. Titus)
;    	Revised by C.K. Thompson for interpolation efficiency  Aug 2006
;Actions:	Add tests for min values in arrays
;		Complete testing - slope, az dependence!

;Input parameters

;	sublut1	subset of the large LUT appropriate for this image
;               size: 5 inertias x 33 slope/az combinations (1 + 4*8)
;               x 3 albedos x 3 elevation/opacity pairs = 2970 points
;		Input is fltarr(5,33,3,6)
;   ??  ISSUE - is it faster to pass these data in a common  ??
;		Note: sublut1 used internally here to avoid issues with repeat calls
;
;	alb	[TES] map albedo for this pixel
;	inert	[TES] map thermal inertia for this pixel
;               Note: interpolation should be done in log(inertia)
;		Nodes are [50.,100.,200.,400.,800.]
;	slope	slope magnitude in deg for this pixel
;	az	slope azimuth in deg clockwise from north
;	opac	opacity - visible band
;	elev	surface elevation of this pixel in km
;		Note: sequence of opac/elev cases is:
;		0.15/-4  0.15/0  0.15/4  0.7/-4  0.7/0  0.7/4


;Output parameters;

;	temp	surface kinetic temp from model for this pixel
;		usage:  temp=fast(....) 

;===============================================================================
;
;
; fast
;
;
;===============================================================================
FUNCTION fast_3_edit,sublut,alb,inert,slope,az,opac,elev

   albi  = [0.10,0.20,0.30]			    ;albedo nodes
   inlog = [3.912,4.605,5.298,5.991,6.685]	    ;nodes in log(inertia) space
   slop  = [0,10,20,30,40]			    ;nodes in slope space
   azim  = [0,45,90,135,180,225,270,315,360]	    ;nodes in slope azimuth space
   opc   = [0.15,0.70]				    ;nodes in opacity
   elv   = [-4,0,4]				    ;nodes in elevation (km)

   ;============================================================================
   ;Set values to endpoints if not in range of parameter space
   ;============================================================================
   IF opac LT 0.15   THEN opac=0.15             ;Use endpoints if need be
   IF opac GT 0.7    THEN opac=0.7

   IF elev LE -4     THEN elev=-3.99            ;Avoid endpoint issues
   IF elev GT 4      THEN elev=4

   IF slope GT 40.    THEN slope=40.
   IF slope LE 0.001 THEN slope=0.001           ;Avoid 0 slope issues

   IF az GT 359.5    THEN az=359.5              ;Avoid 360 issues

   IF alb LT 0.1     THEN alb=0.1
   IF alb GT 0.3     THEN alb=0.3

   IF inert LT 50    THEN inert=50
   IF inert GT 800   THEN inert=800



   ;============================================================================
   ;Inertia interpolation
   ;============================================================================
   inertl   = ALOG(inert)
   min_idx= MIN(WHERE(inlog GE inertl))		;Find relevant indices
   lo_idx = min_idx - 1
   hi_idx = min_idx
   sublut1 = REFORM(sublut[lo_idx,*,*,*]) +              $
               ((inertl-inlog(lo_idx))/0.69315) *        $
                  ((REFORM(sublut[hi_idx,*,*,*]) -      $
                    REFORM(sublut[lo_idx,*,*,*])))

   ;============================================================================
   ; sublut1 is now of size (33,3,6)
   ;============================================================================

   ;============================================================================
   ; albedo interpolation
   ;============================================================================
   min_idx= MIN(WHERE(albi GE alb))		;Get index
   lo_idx = min_idx EQ 0 ? min_idx : min_idx - 1
   hi_idx = min_idx EQ 0 ? min_idx : min_idx
   frac   = min_idx EQ 0 ? min_idx : (alb-albi(min_idx-1))/0.10
   sublut1 = REFORM(sublut1[*,lo_idx,*]) +               $
              frac*((REFORM(sublut1[*,hi_idx,*])-REFORM(sublut1[*,lo_idx,*])))

   ;============================================================================
   ; sublut1 is now of size (33,6)
   ;============================================================================

   ;============================================================================
   ; slope and azimuth interpolation
   ; both must be done together due to the organization of the LUT indices
   ;============================================================================
   slpt  = MIN(WHERE(slop GE slope))     ;Get slope index (min = 1)
   azpt  = MIN(WHERE(azim GE az))        ;Get azimuth index
   IF azpt EQ 0 THEN azpt = 1            ;Deal with 0
   fraz  = (az MOD 45)/45.
   del = 1				 ;index offset for interpolation
   IF az GT 315 THEN del=-7		;deal with 360 wrap issue

   ;============================================================================
   ; interpolate between two az cases for lower slope value
   ; Note - common low angle slope<10 deg case uses tlow for 0 slope
   ;============================================================================
   if slpt EQ 1 THEN begin		;slope is below 10 deg
	tlow = REFORM(sublut1(0,*))	;zero slope case does not use azimuth!
        indl = azpt-8			;to give 10 deg indices below
	goto, next
        endif

;slope exceeds 10 deg
   indl  = (slpt-2)*8 + azpt  		;works?
   tlow  = REFORM(sublut1[indl,*])+ $
           fraz*(REFORM(sublut1[indl+del,*])-REFORM(sublut1[indl,*]))
   
   ;============================================================================
   ; interpolate between two az cases for upper slope value
   ;============================================================================
next:
   indh  = indl+8			;works? 
   thi   = REFORM(sublut1[indh,*])+ $
           fraz*(REFORM(sublut1[indh+del,*])-REFORM(sublut1[indh,*]))

   ;============================================================================
   ; interpolate in slope domain
   ;============================================================================
   frasl = (slope-slop[slpt-1])/10.		
   new3  = tlow+frasl*(thi-tlow)

;print,azpt,slpt,indl,indh,fraz
;print,tlow,thi,new3
;stop
   ;============================================================================
   ; New3 is now of size (6)
   ;============================================================================

   ;============================================================================
   ; opacity and elevation interpolation
   ;============================================================================
   elpt  = MIN(WHERE(elv GE elev))     ;Get elevation index
   IF elpt EQ 0 THEN elpt = 1          ;Deal with min elev
   frel  = (elev-elv[elpt-1])/4.

   ;============================================================================
   ; interpolate in elevation first for lower opacity value
   ;============================================================================
   t1    = new3[elpt-1]+frel*(new3[elpt]-new3[elpt-1])

   ;============================================================================
   ; then for higher opacity value - indices are 3 larger
   ;============================================================================
   t2    = new3[elpt+2]+frel*(new3[elpt+3]-new3[elpt+2])

   ;============================================================================
   ; finally, interpolate in opacity
   ;============================================================================
   frop  = (opac-opc[0])/0.55
   
   RETURN, t1+frop*(t2-t1)

END
;fast.pro
