;******************************************************************************
;
; function write_bsq_slice, lun, nl, slice_idx, slice
;
; Write a slice of ENVI data to a BSQ output file at logical unit LUN.
;
; INPUT:
;	lun         long              Output logical unit
;	nl          int               Number of lines in output image
;	slice_idx   int               Position of the current slice
;	slice       fltarr(ns,nb)     BIL-like slice of data, samples x bands
;
;	     note: slice = reform(full_cube[*,slice_idx,*])
;
; OUTPUT: None
;
; RETURN VALUE:
;	status:   0 success
;	         -1 bad input
;	         -2 IO error on write
;
; KEYWORDS: None.
;
; NOTES:
;	Data Types Accepted: byte, int, uint, long, ulong, float, double
;
; TYPICAL CALL:
;	openw,lun, output_file, /get_lun
;	img = fltarr(ns,nb)
;	for j=0,nl-1 do begin
;		img[*,*] = (Jth BIL-like slice of image)
;		write_bsq_slice, lun, img, nl
;	endfor
;
; USES:
;
; HISTORY:
;	2009 Jan 06  F. Morgan: First version written
;	2009 Aug 18  Replace on_ioerror with catch because on_ioerror might cause
;	             problems executing from save set 
;
;******************************************************************************

function write_bsq_slice, lun, nl, slice_idx, slice


catch, error_status
if (error_status ne 0) then return, -2


; Data size in bytes
tp = size(slice[0], /type)
if ((tp eq 4) or (tp eq 3) or (tp eq 13)) then begin
	ds = 4L    ; long or float or ulong
endif else if ((tp eq 2) or (tp eq 12)) then begin
	ds = 2L    ; int or uint
endif else if (tp eq 1) then begin
	ds = 1L   ; byte
endif else if (tp eq 5) then begin
	ds = 8L    ; double
endif else begin
	return,-1
endelse

; Number of samples, number of bands in the slice
s = size(slice)
if (s[0] ne 2) then return,-1
ns = s[1]
nb = s[2]

fp = ds * slice_idx * ns                ; file ptr at start of first line in slice
file_ptr_increment = ds * nl * ns       ; file ptr increment between bands of slice
for i=0L, nb-1 do begin
	point_lun, lun, fp
	writeu, lun, slice[*,i]
	fp += file_ptr_increment
endfor
return, 0
end
